@@ -136,7 +136,9 @@ int CControlSocket::ResetOperation(int nErrorCode)
 			UnlockCache();
 		}
 		oldOperation = std::move(operations_.back());
-		operations_.pop_back();		
+		operations_.pop_back();
+
+		oldOperation->Reset(nErrorCode);
 	}
 	if (!operations_.empty()) {
 		int ret;
@@ -200,10 +202,7 @@ int CControlSocket::ResetOperation(int nErrorCode)
 						LogMessage(MessageType::Debug_Warning, L"currentServer_ is empty");
 					}
 					else {
-						bool updated = engine_.GetDirectoryCache().UpdateFile(currentServer_, data.remotePath_, data.remoteFile_, true, CDirectoryCache::file, (nErrorCode == FZ_REPLY_OK) ? data.localFileSize_ : -1);
-						if (updated) {
-							SendDirectoryListingNotification(data.remotePath_, false, false);
-						}
+						UpdateCache(data, data.remotePath_, data.remoteFile_, (nErrorCode == FZ_REPLY_OK) ? data.localFileSize_ : -1);
 					}
 				}
 				LogTransferResultMessage(nErrorCode, &data);
@@ -229,6 +228,14 @@ int CControlSocket::ResetOperation(int nErrorCode)
 	return engine_.ResetOperation(nErrorCode);
 }
 
+void CControlSocket::UpdateCache(COpData const &, CServerPath const& serverPath, std::wstring const& remoteFile, int64_t fileSize)
+{
+	bool updated = engine_.GetDirectoryCache().UpdateFile(currentServer_, serverPath, remoteFile, true, CDirectoryCache::file, fileSize);
+	if (updated) {
+		SendDirectoryListingNotification(serverPath, false, false);
+	}
+}
+
 int CControlSocket::DoClose(int nErrorCode)
 {
 	LogMessage(MessageType::Debug_Debug, L"CControlSocket::DoClose(%d)", nErrorCode);
@@ -249,7 +256,7 @@ int CControlSocket::DoClose(int nErrorCode)
 std::wstring CControlSocket::ConvertDomainName(std::wstring const& domain)
 {
 #ifdef FZ_WINDOWS
-	int len = IdnToAscii(IDN_ALLOW_UNASSIGNED, domain.c_str(), domain.size() + 1, 0, 0);
+	int len = IdnToAscii(IDN_ALLOW_UNASSIGNED, domain.c_str(), domain.size() + 1, nullptr, 0);
 	if (!len) {
 		LogMessage(MessageType::Debug_Warning, L"Could not convert domain name");
 		return domain;
@@ -356,6 +363,7 @@ bool CControlSocket::ParsePwdReply(std::wstring reply, bool unquoted, CServerPat
 
 int CControlSocket::CheckOverwriteFile()
 {
+	LogMessage(MessageType::Debug_Debug, L"CControlSocket::CheckOverwriteFile()");
 	if (operations_.empty() || operations_.back()->opId != Command::transfer) {
 		LogMessage(MessageType::Debug_Info, L"CheckOverwriteFile called without active transfer.");
 		return FZ_REPLY_INTERNALERROR;
@@ -867,90 +875,47 @@ void CControlSocket::SendAsyncRequest(CAsyncRequestNotification* pNotification)
 CRealControlSocket::CRealControlSocket(CFileZillaEnginePrivate & engine)
 	: CControlSocket(engine)
 {
-	m_pSocket = new CSocket(engine.GetThreadPool(), this);
+	socket_ = new fz::socket(engine.GetThreadPool(), this);
 
-	m_pBackend = new CSocketBackend(this, *m_pSocket, engine_.GetRateLimiter());
+	m_pBackend = new CSocketBackend(this, *socket_, engine_.GetRateLimiter());
 }
 
 CRealControlSocket::~CRealControlSocket()
 {
-	if (m_pSocket) {
-		m_pSocket->Close();
+	if (socket_) {
+		socket_->close();
 	}
 	if (m_pProxyBackend && m_pProxyBackend != m_pBackend) {
 		delete m_pProxyBackend;
 	}
 	delete m_pBackend;
-	m_pBackend = 0;
+	m_pBackend = nullptr;
 
-	delete m_pSocket;
-	delete[] sendBuffer_;
+	delete socket_;
 }
 
 bool CRealControlSocket::Connected() const
 {
-	return m_pSocket ? (m_pSocket->GetState() == CSocket::connected) : false;
-}
-
-void CRealControlSocket::AppendToSendBuffer(unsigned char const* data, unsigned int len)
-{
-	if (sendBufferSize_ + len > sendBufferCapacity_) {
-		if (sendBufferSize_ + len - sendBufferPos_ <= sendBufferCapacity_) {
-			memmove(sendBuffer_, sendBuffer_ + sendBufferPos_, sendBufferSize_ - sendBufferPos_);
-			sendBufferSize_ -= sendBufferPos_;
-			sendBufferPos_ = 0;
-		}
-		else if (!sendBuffer_) {
-			assert(!sendBufferSize_ && !sendBufferPos_);
-			sendBufferCapacity_ = len;
-			sendBuffer_ = new unsigned char[sendBufferCapacity_];
-		}
-		else {
-			unsigned char *old = sendBuffer_;
-			sendBufferCapacity_ += len;
-			sendBuffer_ = new unsigned char[sendBufferCapacity_];
-			memcpy(sendBuffer_, old + sendBufferPos_, sendBufferSize_ - sendBufferPos_);
-			sendBufferSize_ -= sendBufferPos_;
-			sendBufferPos_ = 0;
-			delete[] old;
-		}
-	}
-
-	memcpy(sendBuffer_ + sendBufferSize_, data, len);
-	sendBufferSize_ += len;
-}
-
-void CRealControlSocket::SendBufferReserve(unsigned int len)
-{
-	if (sendBufferCapacity_ < len) {
-		if (sendBuffer_) {
-			unsigned char *old = sendBuffer_;
-			sendBuffer_ = new unsigned char[sendBufferCapacity_ + len];
-			memcpy(sendBuffer_, old + sendBufferPos_, sendBufferSize_ - sendBufferPos_);
-			sendBufferSize_ -= sendBufferPos_;
-			sendBufferPos_ = 0;
-			sendBufferCapacity_ += len;
-			delete[] old;
-		}
-		else {
-			sendBufferCapacity_ = len;
-			sendBuffer_ = new unsigned char[sendBufferCapacity_];
-		}
-	}
+	return socket_ ? (socket_->get_state() == fz::socket::connected) : false;
 }
 
 int CRealControlSocket::Send(unsigned char const* buffer, unsigned int len)
 {
+	if (!m_pBackend) {
+		LogMessage(MessageType::Debug_Warning, L"Called internal CRealControlSocket::Send without m_pBackend");
+		return FZ_REPLY_INTERNALERROR;
+	}
+
 	SetWait(true);
-	if (sendBufferSize_) {
-		AppendToSendBuffer(buffer, len);
+	if (sendBuffer_) {
+		sendBuffer_.append(buffer, len);
 	}
 	else {
 		int error;
 		int written = m_pBackend->Write(buffer, len, error);
 		if (written < 0) {
 			if (error != EAGAIN) {
-				LogMessage(MessageType::Error, _("Could not write to socket: %s"), CSocket::GetErrorDescription(error));
+				LogMessage(MessageType::Error, _("Could not write to socket: %s"), fz::socket::error_description(error));
 				LogMessage(MessageType::Error, _("Disconnected from server"));
 				return FZ_REPLY_ERROR | FZ_REPLY_DISCONNECTED;
 			}
@@ -962,7 +927,7 @@ int CRealControlSocket::Send(unsigned char const* buffer, unsigned int len)
 		}
 
 		if (static_cast<unsigned int>(written) < len) {
-			AppendToSendBuffer(buffer + written, len - written);
+			sendBuffer_.append(buffer + written, len - written);
 		}
 	}
 
@@ -971,7 +936,7 @@ int CRealControlSocket::Send(unsigned char const* buffer, unsigned int len)
 
 void CRealControlSocket::operator()(fz::event_base const& ev)
 {
-	if (!fz::dispatch<CSocketEvent, CHostAddressEvent>(ev, this,
+	if (!fz::dispatch<fz::socket_event, fz::hostaddress_event>(ev, this,
 		&CRealControlSocket::OnSocketEvent,
 		&CRealControlSocket::OnHostAddress))
 	{
@@ -979,7 +944,7 @@ void CRealControlSocket::operator()(fz::event_base const& ev)
 	}
 }
 
-void CRealControlSocket::OnSocketEvent(CSocketEventSource*, SocketEventType t, int error)
+void CRealControlSocket::OnSocketEvent(fz::socket_event_source*, fz::socket_event_flag t, int error)
 {
 	if (!m_pBackend) {
 		return;
@@ -987,31 +952,32 @@ void CRealControlSocket::OnSocketEvent(CSocketEventSource*, SocketEventType t, i
 
 	switch (t)
 	{
-	case SocketEventType::connection_next:
-		if (error)
-			LogMessage(MessageType::Status, _("Connection attempt failed with \"%s\", trying next address."), CSocket::GetErrorDescription(error));
+	case fz::socket_event_flag::connection_next:
+		if (error) {
+			LogMessage(MessageType::Status, _("Connection attempt failed with \"%s\", trying next address."), fz::socket::error_description(error));
+		}
 		SetAlive();
 		break;
-	case SocketEventType::connection:
+	case fz::socket_event_flag::connection:
 		if (error) {
-			LogMessage(MessageType::Status, _("Connection attempt failed with \"%s\"."), CSocket::GetErrorDescription(error));
+			LogMessage(MessageType::Status, _("Connection attempt failed with \"%s\"."), fz::socket::error_description(error));
 			OnClose(error);
 		}
 		else {
 			if (m_pProxyBackend && !m_pProxyBackend->Detached()) {
 				m_pProxyBackend->Detach();
-				m_pBackend = new CSocketBackend(this, *m_pSocket, engine_.GetRateLimiter());
+				m_pBackend = new CSocketBackend(this, *socket_, engine_.GetRateLimiter());
 			}
 			OnConnect();
 		}
 		break;
-	case SocketEventType::read:
+	case fz::socket_event_flag::read:
 		OnReceive();
 		break;
-	case SocketEventType::write:
+	case fz::socket_event_flag::write:
 		OnSend();
 		break;
-	case SocketEventType::close:
+	case fz::socket_event_flag::close:
 		OnClose(error);
 		break;
 	default:
@@ -1020,7 +986,7 @@ void CRealControlSocket::OnSocketEvent(CSocketEventSource*, SocketEventType t, i
 	}
 }
 
-void CRealControlSocket::OnHostAddress(CSocketEventSource*, std::string const& address)
+void CRealControlSocket::OnHostAddress(fz::socket_event_source*, std::string const& address)
 {
 	if (!m_pBackend) {
 		return;
@@ -1039,15 +1005,12 @@ void CRealControlSocket::OnReceive()
 
 int CRealControlSocket::OnSend()
 {
-	while (sendBufferSize_) {
-		assert(sendBufferPos_ < sendBufferSize_);
-		assert(sendBuffer_);
-
+	while (sendBuffer_) {
 		int error;
-		int written = m_pBackend->Write(sendBuffer_ + sendBufferPos_, sendBufferSize_ - sendBufferPos_, error);
+		int written = m_pBackend->Write(sendBuffer_.get(), sendBuffer_.size(), error);
 		if (written < 0) {
 			if (error != EAGAIN) {
-				LogMessage(MessageType::Error, _("Could not write to socket: %s"), CSocket::GetErrorDescription(error));
+				LogMessage(MessageType::Error, _("Could not write to socket: %s"), fz::socket::error_description(error));
 				if (GetCurrentCommandId() != Command::connect) {
 					LogMessage(MessageType::Error, _("Disconnected from server"));
 				}
@@ -1059,12 +1022,8 @@ int CRealControlSocket::OnSend()
 
 		if (written) {
 			SetActive(CFileZillaEngine::send);
-		}
 
-		sendBufferPos_ += static_cast<int>(written);
-		if (sendBufferPos_ == sendBufferSize_) {
-			sendBufferSize_ = 0;
-			sendBufferPos_ = 0;
+			sendBuffer_.consume(static_cast<size_t>(written));
 		}
 	}
 
@@ -1082,69 +1041,59 @@ void CRealControlSocket::OnClose(int error)
 			LogMessage(messageType, _("Connection closed by server"));
 		}
 		else {
-			LogMessage(messageType, _("Disconnected from server: %s"), CSocket::GetErrorDescription(error));
+			LogMessage(messageType, _("Disconnected from server: %s"), fz::socket::error_description(error));
 		}
 	}
 	DoClose();
 }
 
-int CRealControlSocket::DoConnect(CServer const& server)
+int CRealControlSocket::DoConnect(std::wstring const& host, unsigned int port)
 {
 	SetWait(true);
 
-	if (server.GetEncodingType() == ENCODING_CUSTOM) {
-		LogMessage(MessageType::Debug_Info, L"Using custom encoding: %s", server.GetCustomEncoding());
+	if (currentServer_.GetEncodingType() == ENCODING_CUSTOM) {
+		LogMessage(MessageType::Debug_Info, L"Using custom encoding: %s", currentServer_.GetCustomEncoding());
 	}
 
-	return ContinueConnect();
-}
-
-int CRealControlSocket::ContinueConnect()
-{
-	std::wstring host;
-	unsigned int port = 0;
+	std::wstring real_host;
+	unsigned int real_port = 0;
 
 	const int proxy_type = engine_.GetOptions().GetOptionVal(OPTION_PROXY_TYPE);
 	if (proxy_type > CProxySocket::unknown && proxy_type < CProxySocket::proxytype_count && !currentServer_.GetBypassProxy()) {
 		LogMessage(MessageType::Status, _("Connecting to %s through %s proxy"), currentServer_.Format(ServerFormat::with_optional_port), CProxySocket::Name(static_cast<CProxySocket::ProxyType>(proxy_type)));
 
-		host = engine_.GetOptions().GetOption(OPTION_PROXY_HOST);
-		port = engine_.GetOptions().GetOptionVal(OPTION_PROXY_PORT);
+		real_host = engine_.GetOptions().GetOption(OPTION_PROXY_HOST);
+		real_port = engine_.GetOptions().GetOptionVal(OPTION_PROXY_PORT);
 
 		delete m_pBackend;
-		m_pProxyBackend = new CProxySocket(this, m_pSocket, this);
+		m_pProxyBackend = new CProxySocket(this, socket_, this);
 		m_pBackend = m_pProxyBackend;
 		int res = m_pProxyBackend->Handshake(static_cast<CProxySocket::ProxyType>(proxy_type),
-											ConvertDomainName(currentServer_.GetHost()), currentServer_.GetPort(),
+											ConvertDomainName(host), port,
 											engine_.GetOptions().GetOption(OPTION_PROXY_USER),
 											engine_.GetOptions().GetOption(OPTION_PROXY_PASS));
 
 		if (res != EINPROGRESS) {
-			LogMessage(MessageType::Error, _("Could not start proxy handshake: %s"), CSocket::GetErrorDescription(res));
+			LogMessage(MessageType::Error, _("Could not start proxy handshake: %s"), fz::socket::error_description(res));
 			return FZ_REPLY_DISCONNECTED | FZ_REPLY_ERROR;
 		}
 	}
 	else {
-		if (!operations_.empty() && operations_.back()->opId == Command::connect) {
-			auto & data = static_cast<CConnectOpData&>(*operations_.back());
-			host = data.host_;
-			port = data.port_;
-		}
-		if (host.empty()) {
-			host = currentServer_.GetHost();
-			port = currentServer_.GetPort();
-		}
+		real_host = host;
+		real_port = port;
 	}
 	if (fz::get_address_type(host) == fz::address_type::unknown) {
-		LogMessage(MessageType::Status, _("Resolving address of %s"), host);
+		LogMessage(MessageType::Status, _("Resolving address of %s"), real_host);
 	}
 
-	host = ConvertDomainName(host);
-	int res = m_pSocket->Connect(fz::to_native(host), port);
+	m_closed = false;
+
+	real_host = ConvertDomainName(real_host);
+	int res = socket_->connect(fz::to_native(real_host), real_port);
 
 	// Treat success same as EINPROGRESS, we wait for connect notification in any case
 	if (res && res != EINPROGRESS) {
-		LogMessage(MessageType::Error, _("Could not connect to server: %s"), CSocket::GetErrorDescription(res));
+		LogMessage(MessageType::Error, _("Could not connect to server: %s"), fz::socket::error_description(res));
 		return FZ_REPLY_DISCONNECTED | FZ_REPLY_ERROR; 
 	}
 
@@ -1153,6 +1102,7 @@ int CRealControlSocket::ContinueConnect()
 
 int CRealControlSocket::DoClose(int nErrorCode)
 {
+	LogMessage(MessageType::Debug_Debug, L"CRealControlSocket::DoClose(%d)", nErrorCode);
 	ResetSocket();
 
 	return CControlSocket::DoClose(nErrorCode);
@@ -1160,19 +1110,18 @@ int CRealControlSocket::DoClose(int nErrorCode)
 
 void CRealControlSocket::ResetSocket()
 {
-	m_pSocket->Close();
+	socket_->close();
 
-	sendBufferSize_ = 0;
-	sendBufferPos_ = 0;
+	sendBuffer_.clear();
 
 	if (m_pProxyBackend) {
 		if (m_pProxyBackend != m_pBackend) {
 			delete m_pProxyBackend;
 		}
-		m_pProxyBackend = 0;
+		m_pProxyBackend = nullptr;
 	}
 	delete m_pBackend;
-	m_pBackend = 0;
+	m_pBackend = nullptr;
 }
 
 bool CControlSocket::SetFileExistsAction(CFileExistsNotification *pFileExistsNotification)
@@ -1287,7 +1236,7 @@ bool CControlSocket::SetFileExistsAction(CFileExistsNotification *pFileExistsNot
 
 			int64_t size;
 			bool isLink;
-			if (fz::local_filesys::get_file_info(fz::to_native(data.localFile_), isLink, &size, 0, 0) == fz::local_filesys::file) {
+			if (fz::local_filesys::get_file_info(fz::to_native(data.localFile_), isLink, &size, nullptr, nullptr) == fz::local_filesys::file) {
 				data.localFileSize_ = size;
 			}
 			else {
@@ -1341,7 +1290,7 @@ bool CControlSocket::SetFileExistsAction(CFileExistsNotification *pFileExistsNot
 	return true;
 }
 
-void CControlSocket::CreateLocalDir(std::wstring const & local_file)
+void CControlSocket::CreateLocalDir(std::wstring const& local_file)
 {
 	std::wstring file;
 	CLocalPath local_path(local_file, &file);
