@@ -4,6 +4,8 @@
 #include "socket.h"
 #include "logging_private.h"
 
+#include <libfilezilla/buffer.hpp>
+
 class COpData
 {
 public:
@@ -27,6 +29,9 @@ public:
 
 	virtual int SubcommandResult(int, COpData const&) { return FZ_REPLY_INTERNALERROR; }
 
+	// Called just prior to destructing the operation. Do not push a new operation here.
+	virtual int Reset(int result) { return result; }
+
 	int opState{};
 	Command const opId;
 
@@ -173,7 +178,7 @@ public:
 	virtual void Cancel();
 
 	// Implicit FZ_REPLY_CONTINUE
-	virtual void Connect(CServer const& server) = 0;
+	virtual void Connect(CServer const& server, Credentials const& credentials) = 0;
 	virtual void List(CServerPath const& path = CServerPath(), std::wstring const& subDir = std::wstring(), int flags = 0);
 
 	virtual void FileTransfer(std::wstring const& localFile, CServerPath const& remotePath,
@@ -231,6 +236,7 @@ protected:
 	bool m_closed{};
 
 	virtual int ResetOperation(int nErrorCode);
+	virtual void UpdateCache(COpData const& data, CServerPath const& serverPath, std::wstring const& remoteFile, int64_t fileSize);
 
 	void LogTransferResultMessage(int nErrorCode, CFileTransferOpData *pData);
 
@@ -321,8 +327,7 @@ public:
 	CRealControlSocket(CFileZillaEnginePrivate & engine);
 	virtual ~CRealControlSocket();
 
-	int DoConnect(CServer const& server);
-	virtual int ContinueConnect();
+	int DoConnect(std::wstring const& host, unsigned int port);
 
 	virtual bool Connected() const override;
 
@@ -331,8 +336,8 @@ protected:
 	virtual void ResetSocket();
 
 	virtual void operator()(fz::event_base const& ev) override;
-	void OnSocketEvent(CSocketEventSource* source, SocketEventType t, int error);
-	void OnHostAddress(CSocketEventSource* source, std::string const& address);
+	void OnSocketEvent(fz::socket_event_source* source, fz::socket_event_flag t, int error);
+	void OnHostAddress(fz::socket_event_source* source, std::string const& address);
 
 	virtual void OnConnect();
 	virtual void OnReceive();
@@ -344,17 +349,12 @@ protected:
 		return Send(reinterpret_cast<unsigned char const*>(buffer), len);
 	}
 
-	CSocket* m_pSocket;
+	fz::socket* socket_;
 
 	CBackend* m_pBackend;
 	CProxySocket* m_pProxyBackend{};
 
-	void SendBufferReserve(unsigned int len);
-	void AppendToSendBuffer(unsigned char const* data, unsigned int len);
-	unsigned char* sendBuffer_{};
-	unsigned int sendBufferCapacity_{};
-	unsigned int sendBufferPos_{};
-	unsigned int sendBufferSize_{};
+	fz::buffer sendBuffer_;
 };
 
 #endif
