@@ -2,41 +2,48 @@
 #include "ControlSocket.h"
 #include "directorycache.h"
 #include "engineprivate.h"
-#include "ftpcontrolsocket.h"
-#include "httpcontrolsocket.h"
+#include "ftp/ftpcontrolsocket.h"
+#include "http/httpcontrolsocket.h"
 #include "logging_private.h"
 #include "pathcache.h"
 #include "ratelimiter.h"
-#include "sftpcontrolsocket.h"
+#include "sftp/sftpcontrolsocket.h"
+#if ENABLE_STORJ
+#include "storj/storjcontrolsocket.h"
+#endif
 
 #include <libfilezilla/event_loop.hpp>
 
 #include <algorithm>
 
-fz::mutex CFileZillaEnginePrivate::mutex_;
+fz::mutex CFileZillaEnginePrivate::global_mutex_{false};
 std::vector<CFileZillaEnginePrivate*> CFileZillaEnginePrivate::m_engineList;
 std::atomic_int CFileZillaEnginePrivate::m_activeStatus[2] = {{0}, {0}};
 std::list<CFileZillaEnginePrivate::t_failedLogins> CFileZillaEnginePrivate::m_failedLogins;
 
+namespace {
+unsigned int get_next_engine_id()
+{
+	static std::atomic<int> next_{};
+	return ++next_;
+}
+}
+
 CFileZillaEnginePrivate::CFileZillaEnginePrivate(CFileZillaEngineContext& context, CFileZillaEngine& parent, EngineNotificationHandler& notificationHandler)
 	: event_handler(context.GetEventLoop())
 	, transfer_status_(*this)
 	, notification_handler_(notificationHandler)
+	, m_engine_id(get_next_engine_id())
 	, m_options(context.GetOptions())
 	, m_rateLimiter(context.GetRateLimiter())
 	, directory_cache_(context.GetDirectoryCache())
 	, path_cache_(context.GetPathCache())
 	, parent_(parent)
 	, thread_pool_(context.GetThreadPool())
+	, encoding_converter_(context.GetCustomEncodingConverter())
 {
 	m_engineList.push_back(this);
 
-	{
-		fz::scoped_lock lock(mutex_);
-		static int id = 0;
-		m_engine_id = ++id;
-	}
-
 	m_pLogging = new CLogging(*this);
 
 	{
@@ -63,7 +70,7 @@ CFileZillaEnginePrivate::~CFileZillaEnginePrivate()
 	remove_handler();
 	m_maySendNotificationEvent = false;
 
-	m_pControlSocket.reset();
+	controlSocket_.reset();
 	m_pCurrentCommand.reset();
 
 	// Delete notification list
@@ -81,10 +88,6 @@ CFileZillaEnginePrivate::~CFileZillaEnginePrivate()
 	}
 
 	delete m_pLogging;
-
-	if (m_engineList.empty()) {
-		CSocket::Cleanup(true);
-	}
 }
 
 void CFileZillaEnginePrivate::OnEngineEvent(EngineNotificationType type)
@@ -94,9 +97,6 @@ void CFileZillaEnginePrivate::OnEngineEvent(EngineNotificationType type)
 	case engineCancel:
 		DoCancel();
 		break;
-	case engineTransferEnd:
-		if (m_pControlSocket)
-			m_pControlSocket->TransferEnd();
 	default:
 		break;
 	}
@@ -105,16 +105,18 @@ void CFileZillaEnginePrivate::OnEngineEvent(EngineNotificationType type)
 bool CFileZillaEnginePrivate::IsBusy() const
 {
 	fz::scoped_lock lock(mutex_);
-	return m_pCurrentCommand != 0;
+	return m_pCurrentCommand != nullptr;
 }
 
 bool CFileZillaEnginePrivate::IsConnected() const
 {
 	fz::scoped_lock lock(mutex_);
-	if (!m_pControlSocket)
+	if (!controlSocket_) {
 		return false;
+	}
 
-	return m_pControlSocket->Connected();
+	// FIXME: This is unsafe!
+	return controlSocket_->Connected();
 }
 
 const CCommand *CFileZillaEnginePrivate::GetCurrentCommand() const
@@ -126,10 +128,12 @@ const CCommand *CFileZillaEnginePrivate::GetCurrentCommand() const
 Command CFileZillaEnginePrivate::GetCurrentCommandId() const
 {
 	fz::scoped_lock lock(mutex_);
-	if (!m_pCurrentCommand)
+	if (!m_pCurrentCommand) {
 		return Command::none;
-	else
-		return GetCurrentCommand()->GetId();
+	}
+	else {
+		return m_pCurrentCommand->GetId();
+	}
 }
 
 void CFileZillaEnginePrivate::AddNotification(fz::scoped_lock& lock, CNotification *pNotification)
@@ -215,9 +219,6 @@ int CFileZillaEnginePrivate::ResetOperation(int nErrorCode)
 	fz::scoped_lock lock(mutex_);
 	m_pLogging->LogMessage(MessageType::Debug_Debug, L"CFileZillaEnginePrivate::ResetOperation(%d)", nErrorCode);
 
-	if (nErrorCode & FZ_REPLY_DISCONNECTED)
-		m_lastListDir.clear();
-
 	if (m_pCurrentCommand) {
 		if ((nErrorCode & FZ_REPLY_NOTSUPPORTED) == FZ_REPLY_NOTSUPPORTED) {
 			m_pLogging->LogMessage(MessageType::Error, _("Command not supported by this protocol"));
@@ -234,12 +235,13 @@ int CFileZillaEnginePrivate::ResetOperation(int nErrorCode)
 				if ((nErrorCode & FZ_REPLY_CRITICALERROR) != FZ_REPLY_CRITICALERROR) {
 					++m_retryCount;
 					if (m_retryCount < m_options.GetOptionVal(OPTION_RECONNECTCOUNT) && connectCommand.RetryConnecting()) {
-						unsigned int delay = GetRemainingReconnectDelay(connectCommand.GetServer());
-						if (!delay)
-							delay = 1;
+						fz::duration delay = GetRemainingReconnectDelay(connectCommand.GetServer());
+						if (!delay) {
+							delay = fz::duration::from_seconds(1);
+						}
 						m_pLogging->LogMessage(MessageType::Status, _("Waiting to retry..."));
 						stop_timer(m_retryTimer);
-						m_retryTimer = add_timer(fz::duration::from_milliseconds(delay), true);
+						m_retryTimer = add_timer(delay, true);
 						return FZ_REPLY_WOULDBLOCK;
 					}
 				}
@@ -252,8 +254,9 @@ int CFileZillaEnginePrivate::ResetOperation(int nErrorCode)
 			notification->commandId = m_pCurrentCommand->GetId();
 			AddNotification(notification);
 		}
-		else
+		else {
 			m_nControlSocketError |= nErrorCode;
+		}
 
 		m_pCurrentCommand.reset();
 	}
@@ -285,21 +288,24 @@ unsigned int CFileZillaEnginePrivate::GetNextAsyncRequestNumber()
 // Command handlers
 int CFileZillaEnginePrivate::Connect(CConnectCommand const& command)
 {
-	if (IsConnected())
+	if (IsConnected()) {
 		return FZ_REPLY_ALREADYCONNECTED;
+	}
 
 	m_retryCount = 0;
 
 	// Need to delete before setting m_pCurrentCommand.
 	// The destructor can call CFileZillaEnginePrivate::ResetOperation
 	// which would delete m_pCurrentCommand
-	m_pControlSocket.reset();
+	controlSocket_.reset();
 	m_nControlSocketError = 0;
 
-	if (command.GetServer().GetPort() != CServer::GetDefaultPort(command.GetServer().GetProtocol())) {
-		ServerProtocol protocol = CServer::GetProtocolFromPort(command.GetServer().GetPort(), true);
-		if (protocol != UNKNOWN && protocol != command.GetServer().GetProtocol())
+	auto const& server = command.GetServer();
+	if (server.GetPort() != CServer::GetDefaultPort(server.GetProtocol())) {
+		ServerProtocol protocol = CServer::GetProtocolFromPort(server.GetPort(), true);
+		if (protocol != UNKNOWN && protocol != server.GetProtocol()) {
 			m_pLogging->LogMessage(MessageType::Status, _("Selected port usually in use by a different protocol."));
+		}
 	}
 
 	return ContinueConnect();
@@ -308,9 +314,9 @@ int CFileZillaEnginePrivate::Connect(CConnectCommand const& command)
 int CFileZillaEnginePrivate::Disconnect(CDisconnectCommand const&)
 {
 	int res = FZ_REPLY_OK;
-	if (m_pControlSocket) {
-		res = m_pControlSocket->Disconnect();
-		m_pControlSocket.reset();
+	if (controlSocket_) {
+		res = controlSocket_->Disconnect();
+		controlSocket_.reset();
 	}
 
 	return res;
@@ -323,22 +329,28 @@ int CFileZillaEnginePrivate::List(CListCommand const& command)
 	bool const avoid = (command.GetFlags() & LIST_FLAG_AVOID) != 0;
 
 	if (!refresh && !command.GetPath().empty()) {
-		const CServer* pServer = m_pControlSocket->GetCurrentServer();
-		if (pServer) {
-			CServerPath path(path_cache_.Lookup(*pServer, command.GetPath(), command.GetSubDir()));
-			if (path.empty() && command.GetSubDir().empty())
-				path = command.GetPath();
+		CServer const& server = controlSocket_->GetCurrentServer();
+		if (server) {
+			CServerPath path(path_cache_.Lookup(server, command.GetPath(), command.GetSubDir()));
+			if (path.empty()) {
+				if (command.GetSubDir().empty()) {
+					path = command.GetPath();
+				}
+				else if (server.GetProtocol() == STORJ) {
+                                        path = command.GetPath();
+                                        path.ChangePath(command.GetSubDir());
+                                }
+			}
 			if (!path.empty()) {
 				CDirectoryListing *pListing = new CDirectoryListing;
 				bool is_outdated = false;
-				bool found = directory_cache_.Lookup(*pListing, *pServer, path, true, is_outdated);
+				bool found = directory_cache_.Lookup(*pListing, server, path, true, is_outdated);
 				if (found && !is_outdated) {
-					if (pListing->get_unsure_flags())
+					if (pListing->get_unsure_flags()) {
 						flags |= LIST_FLAG_REFRESH;
+					}
 					else {
 						if (!avoid) {
-							m_lastListDir = pListing->path;
-							m_lastListTime = fz::monotonic_clock::now();
 							CDirectoryListingNotification *pNotification = new CDirectoryListingNotification(pListing->path);
 							AddNotification(pNotification);
 						}
@@ -346,19 +358,22 @@ int CFileZillaEnginePrivate::List(CListCommand const& command)
 						return FZ_REPLY_OK;
 					}
 				}
-				if (is_outdated)
+				if (is_outdated) {
 					flags |= LIST_FLAG_REFRESH;
+				}
 				delete pListing;
 			}
 		}
 	}
 
-	return m_pControlSocket->List(command.GetPath(), command.GetSubDir(), flags);
+	controlSocket_->List(command.GetPath(), command.GetSubDir(), flags);
+	return FZ_REPLY_CONTINUE;
 }
 
 int CFileZillaEnginePrivate::FileTransfer(CFileTransferCommand const& command)
 {
-	return m_pControlSocket->FileTransfer(command.GetLocalFile(), command.GetRemotePath(), command.GetRemoteFile(), command.Download(), command.GetTransferSettings());
+	controlSocket_->FileTransfer(command.GetLocalFile(), command.GetRemotePath(), command.GetRemoteFile(), command.Download(), command.GetTransferSettings());
+	return FZ_REPLY_CONTINUE;
 }
 
 int CFileZillaEnginePrivate::RawCommand(CRawCommand const& command)
@@ -367,7 +382,8 @@ int CFileZillaEnginePrivate::RawCommand(CRawCommand const& command)
 		fz::scoped_lock lock(notification_mutex_);
 		queue_logs_ = false;
 	}
-	return m_pControlSocket->RawCommand(command.GetCommand());
+	controlSocket_->RawCommand(command.GetCommand());
+	return FZ_REPLY_CONTINUE;
 }
 
 int CFileZillaEnginePrivate::Delete(CDeleteCommand& command)
@@ -378,86 +394,40 @@ int CFileZillaEnginePrivate::Delete(CDeleteCommand& command)
 	else {
 		m_pLogging->LogMessage(MessageType::Status, _("Deleting %u files from \"%s\""), static_cast<unsigned int>(command.GetFiles().size()), command.GetPath().GetPath());
 	}
-	return m_pControlSocket->Delete(command.GetPath(), command.ExtractFiles());
+	controlSocket_->Delete(command.GetPath(), command.ExtractFiles());
+	return FZ_REPLY_CONTINUE;
 }
 
 int CFileZillaEnginePrivate::RemoveDir(CRemoveDirCommand const& command)
 {
-	return m_pControlSocket->RemoveDir(command.GetPath(), command.GetSubDir());
+	controlSocket_->RemoveDir(command.GetPath(), command.GetSubDir());
+	return FZ_REPLY_CONTINUE;
 }
 
 int CFileZillaEnginePrivate::Mkdir(CMkdirCommand const& command)
 {
-	return m_pControlSocket->Mkdir(command.GetPath());
+	controlSocket_->Mkdir(command.GetPath());
+	return FZ_REPLY_CONTINUE;
 }
 
 int CFileZillaEnginePrivate::Rename(CRenameCommand const& command)
 {
-	return m_pControlSocket->Rename(command);
+	controlSocket_->Rename(command);
+	return FZ_REPLY_CONTINUE;
 }
 
 int CFileZillaEnginePrivate::Chmod(CChmodCommand const& command)
 {
-	return m_pControlSocket->Chmod(command);
-}
-
-void CFileZillaEnginePrivate::SendDirectoryListingNotification(const CServerPath& path, bool onList, bool modified, bool failed)
-{
-	fz::scoped_lock lock(mutex_);
-
-	assert(m_pControlSocket);
-
-	const CServer* const pOwnServer = m_pControlSocket->GetCurrentServer();
-	assert(pOwnServer);
-
-	m_lastListDir = path;
-
-	if (failed) {
-		AddNotification(new CDirectoryListingNotification(path, false, true));
-		m_lastListTime = fz::monotonic_clock::now();
-
-		// On failed messages, we don't notify other engines
-		return;
-	}
-
-	fz::monotonic_clock changeTime;
-	if (!directory_cache_.GetChangeTime(changeTime, *pOwnServer, path))
-		return;
-
-	AddNotification(new CDirectoryListingNotification(path, !onList));
-	m_lastListTime = changeTime;
-
-	if (!modified)
-		return;
-
-	// Iterate over the other engine, send notification if last listing
-	// directory is the same
-	for (auto & engine : m_engineList) {
-		if (!engine->m_pControlSocket || engine->m_pControlSocket == m_pControlSocket) {
-			continue;
-		}
-
-		CServer const* const pServer = engine->m_pControlSocket->GetCurrentServer();
-		if (!pServer || *pServer != *pOwnServer)
-			continue;
-
-		if (engine->m_lastListDir != path)
-			continue;
-
-		if (engine->m_lastListTime && changeTime <= engine->m_lastListTime)
-			continue;
-
-		engine->m_lastListTime = changeTime;
-		engine->AddNotification(new CDirectoryListingNotification(path, true));
-	}
+	controlSocket_->Chmod(command);
+	return FZ_REPLY_CONTINUE;
 }
 
 void CFileZillaEnginePrivate::RegisterFailedLoginAttempt(const CServer& server, bool critical)
 {
-	fz::scoped_lock lock(mutex_);
+	fz::scoped_lock lock(global_mutex_);
 	std::list<t_failedLogins>::iterator iter = m_failedLogins.begin();
 	while (iter != m_failedLogins.end()) {
-		fz::duration const span = fz::datetime::now() - iter->time;
+		fz::duration const span = fz::monotonic_clock::now() - iter->time;
 		if (span.get_seconds() >= m_options.GetOptionVal(OPTION_RECONNECTDELAY) ||
 			iter->server == server || (!critical && (iter->server.GetHost() == server.GetHost() && iter->server.GetPort() == server.GetPort())))
 		{
@@ -465,41 +435,42 @@ void CFileZillaEnginePrivate::RegisterFailedLoginAttempt(const CServer& server,
 			++iter;
 			m_failedLogins.erase(prev);
 		}
-		else
+		else {
 			++iter;
+		}
 	}
 
 	t_failedLogins failure;
 	failure.server = server;
-	failure.time = fz::datetime::now();
+	failure.time = fz::monotonic_clock::now();
 	failure.critical = critical;
 	m_failedLogins.push_back(failure);
 }
 
-unsigned int CFileZillaEnginePrivate::GetRemainingReconnectDelay(const CServer& server)
+fz::duration CFileZillaEnginePrivate::GetRemainingReconnectDelay(CServer const& server)
 {
-	fz::scoped_lock lock(mutex_);
+	fz::scoped_lock lock(global_mutex_);
 	std::list<t_failedLogins>::iterator iter = m_failedLogins.begin();
 	while (iter != m_failedLogins.end()) {
-		fz::duration const span = fz::datetime::now() - iter->time;
-		const int delay = m_options.GetOptionVal(OPTION_RECONNECTDELAY);
-		if (span.get_seconds() >= delay) {
+		fz::duration const span = fz::monotonic_clock::now() - iter->time;
+		fz::duration const delay = fz::duration::from_seconds(m_options.GetOptionVal(OPTION_RECONNECTDELAY));
+		if (span >= delay) {
 			std::list<t_failedLogins>::iterator prev = iter;
 			++iter;
 			m_failedLogins.erase(prev);
 		}
 		else if (!iter->critical && iter->server.GetHost() == server.GetHost() && iter->server.GetPort() == server.GetPort()) {
-			return delay * 1000 - span.get_milliseconds();
+			return delay - span;
 		}
 		else if (iter->server == server) {
-			return delay * 1000 - span.get_milliseconds();
+			return delay - span;
 		}
 		else {
 			++iter;
 		}
 	}
 
-	return 0;
+	return fz::duration();
 }
 
 void CFileZillaEnginePrivate::OnTimer(fz::timer_id)
@@ -514,11 +485,19 @@ void CFileZillaEnginePrivate::OnTimer(fz::timer_id)
 		return;
 	}
 
-	assert(!IsConnected());
+	controlSocket_.reset();
 
-	m_pControlSocket.reset();
+	int res = ContinueConnect();
+	res |= m_nControlSocketError;
+	m_nControlSocketError = 0;
 
-	ContinueConnect();
+	if (res == FZ_REPLY_CONTINUE) {
+		assert(controlSocket_);
+		controlSocket_->SendNextCommand();
+	}
+	else if (res != FZ_REPLY_WOULDBLOCK) {
+		ResetOperation(res);
+	}
 }
 
 int CFileZillaEnginePrivate::ContinueConnect()
@@ -532,11 +511,11 @@ int CFileZillaEnginePrivate::ContinueConnect()
 
 	const CConnectCommand *pConnectCommand = static_cast<CConnectCommand *>(m_pCurrentCommand.get());
 	const CServer& server = pConnectCommand->GetServer();
-	unsigned int delay = GetRemainingReconnectDelay(server);
+	fz::duration const& delay = GetRemainingReconnectDelay(server);
 	if (delay) {
-		m_pLogging->LogMessage(MessageType::Status, fztranslate("Delaying connection for %d second due to previously failed connection attempt...", "Delaying connection for %d seconds due to previously failed connection attempt...", (delay + 999) / 1000), (delay + 999) / 1000);
+		m_pLogging->LogMessage(MessageType::Status, fztranslate("Delaying connection for %d second due to previously failed connection attempt...", "Delaying connection for %d seconds due to previously failed connection attempt...", (delay.get_milliseconds() + 999) / 1000), (delay.get_milliseconds() + 999) / 1000);
 		stop_timer(m_retryTimer);
-		m_retryTimer = add_timer(fz::duration::from_milliseconds(delay), true);
+		m_retryTimer = add_timer(delay, true);
 		return FZ_REPLY_WOULDBLOCK;
 	}
 
@@ -546,47 +525,52 @@ int CFileZillaEnginePrivate::ContinueConnect()
 	case FTPS:
 	case FTPES:
 	case INSECURE_FTP:
-		m_pControlSocket = std::make_unique<CFtpControlSocket>(*this);
+		controlSocket_ = std::make_unique<CFtpControlSocket>(*this);
 		break;
 	case SFTP:
-		m_pControlSocket = std::make_unique<CSftpControlSocket>(*this);
+		controlSocket_ = std::make_unique<CSftpControlSocket>(*this);
 		break;
 	case HTTP:
 	case HTTPS:
-		m_pControlSocket = std::make_unique<CHttpControlSocket>(*this);
+		controlSocket_ = std::make_unique<CHttpControlSocket>(*this);
 		break;
+#if ENABLE_STORJ
+	case STORJ:
+		controlSocket_ = std::make_unique<CStorjControlSocket>(*this);
+		break;
+#endif
 	default:
-		m_pLogging->LogMessage(MessageType::Debug_Warning, L"Not a valid protocol: %d", server.GetProtocol());
+		m_pLogging->LogMessage(MessageType::Error, _("'%s' is not a supported protocol."), CServer::GetProtocolName(server.GetProtocol()));
 		return FZ_REPLY_SYNTAXERROR|FZ_REPLY_DISCONNECTED;
 	}
 
-	int res = m_pControlSocket->Connect(server);
-	if (m_retryTimer)
-		return FZ_REPLY_WOULDBLOCK;
-
-	return res;
+	controlSocket_->Connect(server, pConnectCommand->GetCredentials());
+	return FZ_REPLY_CONTINUE;
 }
 
 void CFileZillaEnginePrivate::InvalidateCurrentWorkingDirs(const CServerPath& path)
 {
 	fz::scoped_lock lock(mutex_);
 
-	assert(m_pControlSocket);
-	const CServer* const pOwnServer = m_pControlSocket->GetCurrentServer();
-	assert(pOwnServer);
+	assert(controlSocket_);
+	CServer const& ownServer  = controlSocket_->GetCurrentServer();
+	assert(ownServer);
 
 	for (auto & engine : m_engineList) {
-		if (!engine || engine == this)
+		if (!engine || engine == this) {
 			continue;
+		}
 
-		if (!engine->m_pControlSocket)
+		if (!engine->controlSocket_) {
 			continue;
+		}
 
-		const CServer* const pServer = engine->m_pControlSocket->GetCurrentServer();
-		if (!pServer || *pServer != *pOwnServer)
+		CServer const& server = engine->controlSocket_->GetCurrentServer();
+		if (server != ownServer) {
 			continue;
+		}
 
-		engine->m_pControlSocket->InvalidateCurrentWorkingDir(path);
+		engine->controlSocket_->InvalidateCurrentWorkingDir(path);
 	}
 }
 
@@ -613,7 +597,7 @@ int CFileZillaEnginePrivate::CheckCommandPreconditions(CCommand const& command,
 	else if (command.GetId() != Command::connect && command.GetId() != Command::disconnect && !IsConnected()) {
 		return FZ_REPLY_NOTCONNECTED;
 	}
-	else if (command.GetId() == Command::connect && m_pControlSocket) {
+	else if (command.GetId() == Command::connect && controlSocket_) {
 		return FZ_REPLY_ALREADYCONNECTED;
 	}
 	return FZ_REPLY_OK;
@@ -666,27 +650,35 @@ void CFileZillaEnginePrivate::OnCommandEvent()
 			}
 		}
 
-		if (id != Command::disconnect)
+		if (id != Command::disconnect) {
 			res |= m_nControlSocketError;
-		else if (res & FZ_REPLY_DISCONNECTED)
+		}
+		else if (res & FZ_REPLY_DISCONNECTED) {
 			res = FZ_REPLY_OK;
+		}
 		m_nControlSocketError = 0;
 
-		if (res != FZ_REPLY_WOULDBLOCK)
+		if (res == FZ_REPLY_CONTINUE) {
+			assert(controlSocket_);
+			controlSocket_->SendNextCommand();
+		}
+		else if (res != FZ_REPLY_WOULDBLOCK) {
 			ResetOperation(res);
+		}
 	}
 }
 
 void CFileZillaEnginePrivate::DoCancel()
 {
 	fz::scoped_lock lock(mutex_);
-	if (!IsBusy())
+	if (!IsBusy()) {
 		return;
+	}
 
 	if (m_retryTimer) {
 		assert(m_pCurrentCommand && m_pCurrentCommand->GetId() == Command::connect);
 
-		m_pControlSocket.reset();
+		controlSocket_.reset();
 
 		m_pCurrentCommand.reset();
 
@@ -702,19 +694,23 @@ void CFileZillaEnginePrivate::DoCancel()
 		ClearQueuedLogs(true);
 	}
 	else {
-		if (m_pControlSocket)
-			m_pControlSocket->Cancel();
-		else
+		if (controlSocket_) {
+			controlSocket_->Cancel();
+		}
+		else {
 			ResetOperation(FZ_REPLY_CANCELED);
+		}
 	}
 }
 
 bool CFileZillaEnginePrivate::CheckAsyncRequestReplyPreconditions(std::unique_ptr<CAsyncRequestNotification> const& reply)
 {
-	if (!reply)
+	if (!reply) {
 		return false;
-	if (!IsBusy())
+	}
+	if (!IsBusy()) {
 		return false;
+	}
 
 	bool match;
 	{
@@ -722,7 +718,7 @@ bool CFileZillaEnginePrivate::CheckAsyncRequestReplyPreconditions(std::unique_pt
 		match = reply->requestNumber == m_asyncRequestCounter;
 	}
 
-	return match && m_pControlSocket;
+	return match && controlSocket_;
 }
 
 void CFileZillaEnginePrivate::OnSetAsyncRequestReplyEvent(std::unique_ptr<CAsyncRequestNotification> const& reply)
@@ -732,8 +728,8 @@ void CFileZillaEnginePrivate::OnSetAsyncRequestReplyEvent(std::unique_ptr<CAsync
 		return;
 	}
 
-	m_pControlSocket->SetAlive();
-	m_pControlSocket->SetAsyncRequestReply(reply.get());
+	controlSocket_->SetAlive();
+	controlSocket_->SetAsyncRequestReply(reply.get());
 }
 
 int CFileZillaEnginePrivate::Execute(const CCommand &command)
@@ -757,7 +753,7 @@ std::unique_ptr<CNotification> CFileZillaEnginePrivate::GetNextNotification()
 
 	if (m_NotificationList.empty()) {
 		m_maySendNotificationEvent = true;
-		return 0;
+		return nullptr;
 	}
 	std::unique_ptr<CNotification> pNotification(m_NotificationList.front());
 	m_NotificationList.pop_front();
@@ -779,11 +775,13 @@ bool CFileZillaEnginePrivate::SetAsyncRequestReply(std::unique_ptr<CAsyncRequest
 
 bool CFileZillaEnginePrivate::IsPendingAsyncRequestReply(std::unique_ptr<CAsyncRequestNotification> const& pNotification)
 {
-	if (!pNotification)
+	if (!pNotification) {
 		return false;
+	}
 
-	if (!IsBusy())
+	if (!IsBusy()) {
 		return false;
+	}
 
 	fz::scoped_lock lock(notification_mutex_);
 	return pNotification->requestNumber == m_asyncRequestCounter;
@@ -819,14 +817,16 @@ int CFileZillaEnginePrivate::CacheLookup(const CServerPath& path, CDirectoryList
 	// TODO: Possible optimization: Atomically get current server. The cache has its own mutex.
 	fz::scoped_lock lock(mutex_);
 
-	if (!IsConnected())
+	if (!IsConnected()) {
 		return FZ_REPLY_ERROR;
+	}
 
-	assert(m_pControlSocket->GetCurrentServer());
+	assert(controlSocket_->GetCurrentServer());
 
 	bool is_outdated = false;
-	if (!directory_cache_.Lookup(listing, *m_pControlSocket->GetCurrentServer(), path, true, is_outdated))
+	if (!directory_cache_.Lookup(listing, controlSocket_->GetCurrentServer(), path, true, is_outdated)) {
 		return FZ_REPLY_ERROR;
+	}
 
 	return FZ_REPLY_OK;
 }
@@ -834,8 +834,9 @@ int CFileZillaEnginePrivate::CacheLookup(const CServerPath& path, CDirectoryList
 int CFileZillaEnginePrivate::Cancel()
 {
 	fz::scoped_lock lock(mutex_);
-	if (!IsBusy())
+	if (!IsBusy()) {
 		return FZ_REPLY_OK;
+	}
 
 	send_event<CFileZillaEngineEvent>(engineCancel);
 	return FZ_REPLY_WOULDBLOCK;
@@ -873,8 +874,9 @@ void CTransferStatusManager::Reset()
 void CTransferStatusManager::Init(int64_t totalSize, int64_t startOffset, bool list)
 {
 	fz::scoped_lock lock(mutex_);
-	if (startOffset < 0)
+	if (startOffset < 0) {
 		startOffset = 0;
+	}
 
 	status_ = CTransferStatus(totalSize, startOffset, list);
 	currentOffset_ = 0;
@@ -883,8 +885,9 @@ void CTransferStatusManager::Init(int64_t totalSize, int64_t startOffset, bool l
 void CTransferStatusManager::SetStartTime()
 {
 	fz::scoped_lock lock(mutex_);
-	if (!status_)
+	if (!status_) {
 		return;
+	}
 
 	status_.started = fz::datetime::now();
 }
@@ -892,15 +895,16 @@ void CTransferStatusManager::SetStartTime()
 void CTransferStatusManager::SetMadeProgress()
 {
 	fz::scoped_lock lock(mutex_);
-	if (!status_)
+	if (!status_) {
 		return;
+	}
 
 	status_.madeProgress = true;
 }
 
 void CTransferStatusManager::Update(int64_t transferredBytes)
 {
-	CNotification* notification = 0;
+	CNotification* notification = nullptr;
 
 	{
 		int64_t oldOffset = currentOffset_.fetch_add(transferredBytes);
