@@ -5,28 +5,30 @@
 #include "../servercapabilities.h"
 #include "../tlssocket.h"
 
-CFtpLogonOpData::CFtpLogonOpData(CFtpControlSocket& controlSocket, CServer const& server)
-    : CConnectOpData(server), CFtpOpData(controlSocket)
+CFtpLogonOpData::CFtpLogonOpData(CFtpControlSocket& controlSocket, Credentials const& credentials)
+    : COpData(Command::connect)
+	, CFtpOpData(controlSocket)
+	, credentials_(credentials)
 {
 	for (int i = 0; i < LOGON_DONE; ++i) {
 		neededCommands[i] = 1;
 	}
 
-	if (server.GetProtocol() != FTPES && server.GetProtocol() != FTP) {
+	if (currentServer_.GetProtocol() != FTPES && currentServer_.GetProtocol() != FTP) {
 		neededCommands[LOGON_AUTH_TLS] = 0;
 		neededCommands[LOGON_AUTH_SSL] = 0;
 		neededCommands[LOGON_AUTH_WAIT] = 0;
-		if (server.GetProtocol() != FTPS) {
+		if (currentServer_.GetProtocol() != FTPS) {
 			neededCommands[LOGON_PBSZ] = 0;
 			neededCommands[LOGON_PROT] = 0;
 		}
 	}
-	if (server.GetPostLoginCommands().empty()) {
+	if (currentServer_.GetPostLoginCommands().empty()) {
 		neededCommands[LOGON_CUSTOMCOMMANDS] = 0;
 	}
 
-	const CharsetEncoding encoding = server.GetEncodingType();
-	if (encoding == ENCODING_AUTO && CServerCapabilities::GetCapability(server, utf8_command) != no) {
+	const CharsetEncoding encoding = currentServer_.GetEncodingType();
+	if (encoding == ENCODING_AUTO && CServerCapabilities::GetCapability(currentServer_, utf8_command) != no) {
 		controlSocket_.m_useUTF8 = true;
 	}
 	else if (encoding == ENCODING_UTF8) {
@@ -42,15 +44,17 @@ int CFtpLogonOpData::Send()
 	{
 	case LOGON_CONNECT:
 	    {
-		    if (!GetLoginSequence()) {
+			// Do not use FTP proxy if generic proxy is set
+			int const generic_proxy_type = engine_.GetOptions().GetOptionVal(OPTION_PROXY_TYPE);
+			if ((generic_proxy_type <= CProxySocket::unknown || generic_proxy_type >= CProxySocket::proxytype_count) && !currentServer_.GetBypassProxy()) {
+				ftp_proxy_type_ = engine_.GetOptions().GetOptionVal(OPTION_FTP_PROXY_TYPE);
+			}
+		
+			if (!PrepareLoginSequence()) {
 				return FZ_REPLY_INTERNALERROR;
 			}
-
-			// Do not use FTP proxy if generic proxy is set
-			int generic_proxy_type = engine_.GetOptions().GetOptionVal(OPTION_PROXY_TYPE);
-			if ((generic_proxy_type <= CProxySocket::unknown || generic_proxy_type >= CProxySocket::proxytype_count) &&
-			    (ftp_proxy_type = engine_.GetOptions().GetOptionVal(OPTION_FTP_PROXY_TYPE)) && !server_.GetBypassProxy())
-			{
+			
+			if (ftp_proxy_type_) {
 				host_ = engine_.GetOptions().GetOption(OPTION_FTP_PROXY_HOST);
 
 				size_t pos = -1;
@@ -89,16 +93,15 @@ int CFtpLogonOpData::Send()
 					return FZ_REPLY_DISCONNECTED | FZ_REPLY_CRITICALERROR;
 				}
 
-				LogMessage(MessageType::Status, _("Connecting to %s through %s proxy"), server_.Format(ServerFormat::with_optional_port), L"FTP"); // @translator: Connecting to ftp.example.com through SOCKS5 proxy
+				LogMessage(MessageType::Status, _("Connecting to %s through %s proxy"), currentServer_.Format(ServerFormat::with_optional_port, credentials_), L"FTP"); // @translator: Connecting to ftp.example.com through SOCKS5 proxy
 			}
 			else {
-				ftp_proxy_type = 0;
-				host_ = server_.GetHost();
-				port_ = server_.GetPort();
+				host_ = currentServer_.GetHost();
+				port_ = currentServer_.GetPort();
 			}
 
 			opState = LOGON_WELCOME;
-			return controlSocket_.DoConnect(server_);
+			return controlSocket_.DoConnect(host_, port_);
 	    }
 	case LOGON_AUTH_WAIT:
 		LogMessage(MessageType::Debug_Info, L"LogonSend() called during LOGON_AUTH_WAIT, ignoring");
@@ -115,7 +118,7 @@ int CFtpLogonOpData::Send()
 			switch (cmd.type)
 			{
 			case loginCommandType::user:
-				if (currentServer_.GetLogonType() == INTERACTIVE) {
+				if (credentials_.logonType_ == LogonType::interactive) {
 					waitChallenge = true;
 					challenge.clear();
 				}
@@ -130,6 +133,7 @@ int CFtpLogonOpData::Send()
 				if (!challenge.empty()) {
 					CInteractiveLoginNotification *pNotification = new CInteractiveLoginNotification(CInteractiveLoginNotification::interactive, challenge, false);
 					pNotification->server = currentServer_;
+					pNotification->credentials = credentials_;
 					challenge.clear();
 
 					controlSocket_.SendAsyncRequest(pNotification);
@@ -138,11 +142,11 @@ int CFtpLogonOpData::Send()
 				}
 
 				if (cmd.command.empty()) {
-					return controlSocket_.SendCommand(L"PASS " + currentServer_.GetPass(), true);
+					return controlSocket_.SendCommand(L"PASS " + credentials_.GetPass(), true);
 				}
 				else {
 					std::wstring c = cmd.command;
-					std::wstring pass = currentServer_.GetPass();
+					std::wstring pass = credentials_.GetPass();
 					fz::replace_substrings(pass, L"%", L"%%");
 					fz::replace_substrings(c, L"%p", pass);
 					fz::replace_substrings(c, L"%%", L"%");
@@ -151,7 +155,7 @@ int CFtpLogonOpData::Send()
 				break;
 			case loginCommandType::account:
 				if (cmd.command.empty()) {
-					return controlSocket_.SendCommand(L"ACCT " + currentServer_.GetAccount());
+					return controlSocket_.SendCommand(L"ACCT " + credentials_.account_);
 				}
 				else {
 					return controlSocket_.SendCommand(cmd.command);
@@ -246,7 +250,7 @@ int CFtpLogonOpData::ParseResponse()
 			assert(!controlSocket_.m_pTlsSocket);
 			delete controlSocket_.m_pBackend;
 
-			controlSocket_.m_pTlsSocket = new CTlsSocket(&controlSocket_, *controlSocket_.m_pSocket, &controlSocket_);
+			controlSocket_.m_pTlsSocket = new CTlsSocket(&controlSocket_, *controlSocket_.socket_, &controlSocket_);
 			controlSocket_.m_pBackend = controlSocket_.m_pTlsSocket;
 
 			if (!controlSocket_.m_pTlsSocket->Init()) {
@@ -274,7 +278,7 @@ int CFtpLogonOpData::ParseResponse()
 				if (!user.empty() && (user.front() == ' ' || user.back() == ' ')) {
 					LogMessage(MessageType::Status, _("Check your login credentials. The entered username starts or ends with a space character."));
 				}
-				auto const pw = currentServer_.GetPass();
+				auto const pw = credentials_.GetPass();
 				if (!pw.empty() && (pw.front() == ' ' || pw.back() == ' ')) {
 					LogMessage(MessageType::Status, _("Check your login credentials. The entered password starts or ends with a space character."));
 				}
@@ -287,14 +291,14 @@ int CFtpLogonOpData::ParseResponse()
 				if (!fz::str_is_ascii(currentServer_.GetUser())) {
 					asciiOnly = false;
 				}
-				if (!fz::str_is_ascii(currentServer_.GetPass())) {
+				if (!fz::str_is_ascii(credentials_.GetPass())) {
 					asciiOnly = false;
 				}
-				if (!fz::str_is_ascii(currentServer_.GetAccount())) {
+				if (!fz::str_is_ascii(credentials_.account_)) {
 					asciiOnly = false;
 				}
 				if (!asciiOnly) {
-					if (ftp_proxy_type) {
+					if (ftp_proxy_type_) {
 						LogMessage(MessageType::Status, _("Login data contains non-ASCII characters and server might not be UTF-8 aware. Cannot fall back to local charset since using proxy."));
 						int error = FZ_REPLY_DISCONNECTED | FZ_REPLY_ERROR;
 						if (cmd.type == loginCommandType::pass && code == 5) {
@@ -304,7 +308,7 @@ int CFtpLogonOpData::ParseResponse()
 					}
 					LogMessage(MessageType::Status, _("Login data contains non-ASCII characters and server might not be UTF-8 aware. Trying local charset."));
 					controlSocket_.m_useUTF8 = false;
-					if (!GetLoginSequence()) {
+					if (!PrepareLoginSequence()) {
 						int error = FZ_REPLY_DISCONNECTED | FZ_REPLY_ERROR;
 						if (cmd.type == loginCommandType::pass && code == 5) {
 							error |= FZ_REPLY_PASSWORDFAILED;
@@ -330,7 +334,7 @@ int CFtpLogonOpData::ParseResponse()
 		}
 		else if (code == 3 && loginSequence.empty()) {
 			LogMessage(MessageType::Error, _("Login sequence fully executed yet not logged in, aborting."));
-			if (cmd.type == loginCommandType::pass && currentServer_.GetAccount().empty()) {
+			if (cmd.type == loginCommandType::pass && credentials_.account_.empty()) {
 				LogMessage(MessageType::Error, _("Server might require an account. Try specifying an account using the Site Manager"));
 			}
 			return FZ_REPLY_CRITICALERROR | FZ_REPLY_DISCONNECTED;
@@ -553,11 +557,11 @@ int CFtpLogonOpData::ParseResponse()
 	return FZ_REPLY_CONTINUE;
 }
 
-bool CFtpLogonOpData::GetLoginSequence()
+bool CFtpLogonOpData::PrepareLoginSequence()
 {
 	loginSequence.clear();
 
-	if (!ftp_proxy_type) {
+	if (!ftp_proxy_type_) {
 		// User
 		t_loginCommand cmd = {false, false, loginCommandType::user, L""};
 		loginSequence.push_back(cmd);
@@ -569,13 +573,13 @@ bool CFtpLogonOpData::GetLoginSequence()
 		loginSequence.push_back(cmd);
 
 		// Optional account
-		if (!server_.GetAccount().empty()) {
+		if (!credentials_.account_.empty()) {
 			cmd.hide_arguments = false;
 			cmd.type = loginCommandType::account;
 			loginSequence.push_back(cmd);
 		}
 	}
-	else if (ftp_proxy_type == 1) {
+	else if (ftp_proxy_type_ == 1) {
 		std::wstring const proxyUser = engine_.GetOptions().GetOption(OPTION_FTP_PROXY_USER);
 		if (!proxyUser.empty()) {
 			// Proxy logon (if credendials are set)
@@ -587,7 +591,7 @@ bool CFtpLogonOpData::GetLoginSequence()
 			loginSequence.push_back(cmd);
 		}
 		// User@host
-		t_loginCommand cmd = {false, false, loginCommandType::user, fz::sprintf(L"USER %s@%s", server_.GetUser(), server_.Format(ServerFormat::with_optional_port))};
+		t_loginCommand cmd = {false, false, loginCommandType::user, fz::sprintf(L"USER %s@%s", currentServer_.GetUser(), currentServer_.Format(ServerFormat::with_optional_port, credentials_))};
 		loginSequence.push_back(cmd);
 
 		// Password
@@ -598,13 +602,13 @@ bool CFtpLogonOpData::GetLoginSequence()
 		loginSequence.push_back(cmd);
 
 		// Optional account
-		if (!server_.GetAccount().empty()) {
+		if (!credentials_.account_.empty()) {
 			cmd.hide_arguments = false;
 			cmd.type = loginCommandType::account;
 			loginSequence.push_back(cmd);
 		}
 	}
-	else if (ftp_proxy_type == 2 || ftp_proxy_type == 3) {
+	else if (ftp_proxy_type_ == 2 || ftp_proxy_type_ == 3) {
 		std::wstring const proxyUser = engine_.GetOptions().GetOption(OPTION_FTP_PROXY_USER);
 		if (!proxyUser.empty()) {
 			// Proxy logon (if credendials are set)
@@ -618,11 +622,11 @@ bool CFtpLogonOpData::GetLoginSequence()
 
 		// Site or Open
 		t_loginCommand cmd = {false, false, loginCommandType::user, L""};
-		if (ftp_proxy_type == 2) {
-			cmd.command = L"SITE " + server_.Format(ServerFormat::with_optional_port);
+		if (ftp_proxy_type_ == 2) {
+			cmd.command = L"SITE " + currentServer_.Format(ServerFormat::with_optional_port, credentials_);
 		}
 		else {
-			cmd.command = L"OPEN " + server_.Format(ServerFormat::with_optional_port);
+			cmd.command = L"OPEN " + currentServer_.Format(ServerFormat::with_optional_port, credentials_);
 		}
 		loginSequence.push_back(cmd);
 
@@ -638,18 +642,18 @@ bool CFtpLogonOpData::GetLoginSequence()
 		loginSequence.push_back(cmd);
 
 		// Optional account
-		if (!server_.GetAccount().empty()) {
+		if (!credentials_.account_.empty()) {
 			cmd.hide_arguments = false;
 			cmd.type = loginCommandType::account;
 			loginSequence.push_back(cmd);
 		}
 	}
-	else if (ftp_proxy_type == 4) {
+	else if (ftp_proxy_type_ == 4) {
 		std::wstring proxyUser = engine_.GetOptions().GetOption(OPTION_FTP_PROXY_USER);
 		std::wstring proxyPass = engine_.GetOptions().GetOption(OPTION_FTP_PROXY_PASS);
-		std::wstring host = server_.Format(ServerFormat::with_optional_port);
-		std::wstring user = server_.GetUser();
-		std::wstring account = server_.GetAccount();
+		std::wstring host = currentServer_.Format(ServerFormat::with_optional_port, credentials_);
+		std::wstring user = currentServer_.GetUser();
+		std::wstring account = credentials_.account_;
 		fz::replace_substrings(proxyUser, L"%", L"%%");
 		fz::replace_substrings(proxyPass, L"%", L"%%");
 		fz::replace_substrings(host, L"%", L"%%");
