@@ -14,7 +14,7 @@
 #include "remote_recursive_operation.h"
 #include "themeprovider.h"
 
-#include "uri.h"
+#include <libfilezilla/uri.hpp>
 
 #include <wx/clipbrd.h>
 
@@ -57,8 +57,9 @@ public:
 		int flags = 0;
 		wxTreeItemId hit = m_pRemoteTreeView->HitTest(point, flags);
 
-		if (flags & (wxTREE_HITTEST_ABOVE | wxTREE_HITTEST_BELOW | wxTREE_HITTEST_NOWHERE | wxTREE_HITTEST_TOLEFT | wxTREE_HITTEST_TORIGHT))
+		if (flags & (wxTREE_HITTEST_ABOVE | wxTREE_HITTEST_BELOW | wxTREE_HITTEST_NOWHERE | wxTREE_HITTEST_TOLEFT | wxTREE_HITTEST_TORIGHT)) {
 			return wxTreeItemId();
+		}
 
 		return hit;
 	}
@@ -71,32 +72,34 @@ public:
 			return def;
 
 		wxTreeItemId hit = GetHit(wxPoint(x, y));
-		if (!hit)
+		if (!hit) {
 			return wxDragNone;
+		}
 
 		CServerPath path = m_pRemoteTreeView->GetPathFromItem(hit);
-		if (path.empty())
+		if (path.empty()) {
 			return wxDragNone;
+		}
 
-		if (!GetData())
+		if (!GetData()) {
 			return wxDragError;
+		}
 
 		CDragDropManager* pDragDropManager = CDragDropManager::Get();
-		if (pDragDropManager)
+		if (pDragDropManager) {
 			pDragDropManager->pDropTarget = m_pRemoteTreeView;
+		}
 
-		if (m_pDataObject->GetReceivedFormat() == m_pFileDataObject->GetFormat())
+		if (m_pDataObject->GetReceivedFormat() == m_pFileDataObject->GetFormat()) {
 			m_pRemoteTreeView->m_state.UploadDroppedFiles(m_pFileDataObject, path, false);
-		else
-		{
-			if (m_pRemoteDataObject->GetProcessId() != (int)wxGetProcessId())
-			{
+		}
+		else {
+			if (m_pRemoteDataObject->GetProcessId() != (int)wxGetProcessId()) {
 				wxMessageBoxEx(_("Drag&drop between different instances of FileZilla has not been implemented yet."));
 				return wxDragNone;
 			}
 
-			if (!m_pRemoteTreeView->m_state.GetServer() || !m_pRemoteDataObject->GetServer().EqualsNoPass(*m_pRemoteTreeView->m_state.GetServer()))
-			{
+			if (!m_pRemoteTreeView->m_state.GetServer() || m_pRemoteDataObject->GetServer().server != m_pRemoteTreeView->m_state.GetServer().server) {
 				wxMessageBoxEx(_("Drag&drop between different servers has not been implemented yet."));
 				return wxDragNone;
 			}
@@ -219,8 +222,6 @@ protected:
 	wxDataObjectComposite* m_pDataObject;
 };
 
-IMPLEMENT_CLASS(CRemoteTreeView, wxTreeCtrlEx)
-
 BEGIN_EVENT_TABLE(CRemoteTreeView, wxTreeCtrlEx)
 EVT_TREE_ITEM_EXPANDING(wxID_ANY, CRemoteTreeView::OnItemExpanding)
 EVT_TREE_SEL_CHANGED(wxID_ANY, CRemoteTreeView::OnSelectionChanged)
@@ -258,7 +259,6 @@ CRemoteTreeView::CRemoteTreeView(wxWindow* parent, wxWindowID id, CState& state,
 	UpdateSortMode();
 	RegisterOption(OPTION_FILELIST_NAMESORT);
 
-	m_busy = false;
 	m_pQueue = pQueue;
 	AddRoot(_T(""));
 	m_ExpandAfterList = wxTreeItemId();
@@ -320,12 +320,13 @@ void CRemoteTreeView::SetDirectoryListing(std::shared_ptr<CDirectoryListing> con
 		return;
 	}
 
+	bool const wasEmpty = GetChildrenCount(GetRootItem()) == 0;
+
 #ifndef __WXMSW__
 	Freeze();
 #endif
 	wxTreeItemId parent = MakeParent(pListing->path, !modified);
-	if (!parent)
-	{
+	if (!parent) {
 		m_busy = false;
 #ifndef __WXMSW__
 		Thaw();
@@ -333,19 +334,21 @@ void CRemoteTreeView::SetDirectoryListing(std::shared_ptr<CDirectoryListing> con
 		return;
 	}
 
-	if (!IsExpanded(parent) && parent != m_ExpandAfterList)
-	{
+	if (wasEmpty && GetItemParent(parent) == GetRootItem() && !m_ExpandAfterList) {
+		m_ExpandAfterList = parent;
+	}
+
+	if (!IsExpanded(parent) && parent != m_ExpandAfterList) {
 		DeleteChildren(parent);
 		CFilterManager filter;
-		if (HasSubdirs(*pListing, filter))
+		if (HasSubdirs(*pListing, filter)) {
 			AppendItem(parent, _T(""), -1, -1);
+		}
 	}
-	else
-	{
+	else {
 		RefreshItem(parent, *pListing, !modified);
 
-		if (m_ExpandAfterList == parent)
-		{
+		if (m_ExpandAfterList == parent) {
 #ifndef __WXMSW__
 			// Prevent CalculatePositions from being called
 			wxGenericTreeItem *anchor = m_anchor;
@@ -364,11 +367,13 @@ void CRemoteTreeView::SetDirectoryListing(std::shared_ptr<CDirectoryListing> con
 #ifndef __WXMSW__
 	Thaw();
 #endif
-	if (!modified)
+	if (!modified) {
 		SafeSelectItem(parent);
+	}
 #ifndef __WXMSW__
-	else
+	else {
 		Refresh();
+	}
 #endif
 
 	m_busy = false;
@@ -376,11 +381,11 @@ void CRemoteTreeView::SetDirectoryListing(std::shared_ptr<CDirectoryListing> con
 
 wxTreeItemId CRemoteTreeView::MakeParent(CServerPath path, bool select)
 {
-	std::vector<wxString> pieces;
+	std::vector<std::wstring> pieces;
 	pieces.reserve(path.SegmentCount() + 1);
 	while (path.HasParent()) {
 		pieces.push_back(path.GetLastSegment());
-		path = path.GetParent();
+		path.MakeParent();
 	}
 	wxASSERT(!path.GetPath().empty());
 	pieces.push_back(path.GetPath());
@@ -388,9 +393,9 @@ wxTreeItemId CRemoteTreeView::MakeParent(CServerPath path, bool select)
 	const wxTreeItemId root = GetRootItem();
 	wxTreeItemId parent = root;
 
-	for (std::vector<wxString>::const_reverse_iterator iter = pieces.rbegin(); iter != pieces.rend(); ++iter) {
+	for (auto iter = pieces.crbegin(); iter != pieces.crend(); ++iter) {
 		if (iter != pieces.rbegin()) {
-			path.AddSegment(iter->ToStdWstring());
+			path.AddSegment(*iter);
 		}
 
 		wxTreeItemIdValue cookie;
@@ -398,13 +403,15 @@ wxTreeItemId CRemoteTreeView::MakeParent(CServerPath path, bool select)
 		if (child && GetItemText(child).empty()) {
 			Delete(child);
 			child = wxTreeItemId();
-			if (parent != root)
+			if (parent != root) {
 				ListExpand(parent);
+			}
 		}
 		for (child = GetFirstChild(parent, cookie); child; child = GetNextSibling(child)) {
 			const wxString& text = GetItemText(child);
-			if (text == *iter)
+			if (text == *iter) {
 				break;
+			}
 		}
 		if (!child) {
 			CDirectoryListing listing;
@@ -421,8 +428,9 @@ wxTreeItemId CRemoteTreeView::MakeParent(CServerPath path, bool select)
 
 			auto nextIter = iter;
 			++nextIter;
-			if (nextIter != pieces.rend())
+			if (nextIter != pieces.rend()) {
 				DisplayItem(child, listing);
+			}
 		}
 		if (select && iter != pieces.rbegin()) {
 #ifndef __WXMSW__
@@ -444,11 +452,12 @@ wxTreeItemId CRemoteTreeView::MakeParent(CServerPath path, bool select)
 
 wxBitmap CRemoteTreeView::CreateIcon(int index, const wxString& overlay)
 {
+	auto * imageList = GetSystemImageList();
 #ifdef __WXMSW__
 	// Need to use wxImageList::GetIcon, wxImageList::GetBitmap kills the alpha channel on MSW...
-	wxBitmap bmp = GetSystemImageList()->GetIcon(index);
+	wxBitmap bmp = imageList ? imageList->GetIcon(index) : wxBitmap();
 #else
-	wxBitmap bmp = GetSystemImageList()->GetBitmap(index);
+	wxBitmap bmp = imageList ? imageList->GetBitmap(index) : wxBitmap();
 #endif
 	if (!overlay.empty()) {
 		wxBitmap unknown = CThemeProvider::Get()->CreateBitmap(overlay, wxART_OTHER, bmp.GetScaledSize());
@@ -464,6 +473,10 @@ wxBitmap CRemoteTreeView::CreateIcon(int index, const wxString& overlay)
 
 void CRemoteTreeView::CreateImageList()
 {
+	if (!GetSystemImageList()) {
+		return;
+	}
+
 	// Normal directory
 	int index = GetIconIndex(iconType::dir, _T("{78013B9C-3532-4fe1-A418-5CD1955127CC}"), false);
 	wxBitmap dirIcon = CreateIcon(index);
@@ -489,20 +502,23 @@ void CRemoteTreeView::CreateImageList()
 
 bool CRemoteTreeView::HasSubdirs(const CDirectoryListing& listing, const CFilterManager& filter)
 {
-	if (!listing.has_dirs())
+	if (!listing.has_dirs()) {
 		return false;
+	}
 
-	if (!filter.HasActiveFilters())
+	if (!filter.HasActiveFilters()) {
 		return true;
+	}
 
 	const wxString path = listing.path.GetPath();
-	for (unsigned int i = 0; i < listing.GetCount(); i++)
-	{
-		if (!listing[i].is_dir())
+	for (unsigned int i = 0; i < listing.GetCount(); ++i) {
+		if (!listing[i].is_dir()) {
 			continue;
+		}
 
-		if (filter.FilenameFiltered(listing[i].name, path, true, -1, false, 0, listing[i].time))
+		if (filter.FilenameFiltered(listing[i].name, path, true, -1, false, 0, listing[i].time)) {
 			continue;
+		}
 
 		return true;
 	}
@@ -518,11 +534,13 @@ void CRemoteTreeView::DisplayItem(wxTreeItemId parent, const CDirectoryListing&
 
 	CFilterDialog filter;
 	for (unsigned int i = 0; i < listing.GetCount(); ++i) {
-		if (!listing[i].is_dir())
+		if (!listing[i].is_dir()) {
 			continue;
+		}
 
-		if (filter.FilenameFiltered(listing[i].name, path, true, -1, false, 0, listing[i].time))
+		if (filter.FilenameFiltered(listing[i].name, path, true, -1, false, 0, listing[i].time)) {
 			continue;
+		}
 
 		std::wstring const& name = listing[i].name;
 		CServerPath subdir = listing.path;
@@ -534,8 +552,9 @@ void CRemoteTreeView::DisplayItem(wxTreeItemId parent, const CDirectoryListing&
 			wxTreeItemId child = AppendItem(parent, name, 0, 2, 0);
 			SetItemImages(child, false);
 
-			if (HasSubdirs(subListing, filter))
+			if (HasSubdirs(subListing, filter)) {
 				AppendItem(child, _T(""), -1, -1);
+			}
 		}
 		else {
 			wxTreeItemId child = AppendItem(parent, name, 1, 3, 0);
@@ -677,15 +696,16 @@ void CRemoteTreeView::RefreshItem(wxTreeItemId parent, const CDirectoryListing&
 
 void CRemoteTreeView::OnItemExpanding(wxTreeEvent& event)
 {
-	if (m_busy)
+	if (m_busy) {
 		return;
+	}
 
 	wxTreeItemId item = event.GetItem();
-	if (!item)
+	if (!item) {
 		return;
+	}
 
-	if (!ListExpand(item))
-	{
+	if (!ListExpand(item)) {
 		event.Veto();
 		return;
 	}
@@ -718,10 +738,12 @@ void CRemoteTreeView::SetItemImages(wxTreeItemId item, bool unknown)
 
 void CRemoteTreeView::OnSelectionChanged(wxTreeEvent& event)
 {
-	if (event.GetItem() != m_ExpandAfterList)
+	if (event.GetItem() != m_ExpandAfterList) {
 		m_ExpandAfterList = wxTreeItemId();
-	if (m_busy)
+	}
+	if (m_busy) {
 		return;
+	}
 
 	if (!m_state.IsRemoteIdle(true)) {
 		wxBell();
@@ -729,13 +751,15 @@ void CRemoteTreeView::OnSelectionChanged(wxTreeEvent& event)
 	}
 
 	wxTreeItemId item = event.GetItem();
-	if (!item)
+	if (!item) {
 		return;
+	}
 
 	const CServerPath path = GetPathFromItem(item);
 	wxASSERT(!path.empty());
-	if (path.empty())
+	if (path.empty()) {
 		return;
+	}
 
 	m_state.ChangeRemoteDir(path);
 }
@@ -783,27 +807,29 @@ void CRemoteTreeView::OnBeginDrag(wxTreeEvent& event)
 	}
 
 	const wxTreeItemId& item = event.GetItem();
-	if (!item)
+	if (!item) {
 		return;
+	}
 
 	CServerPath path = GetPathFromItem(item);
-	if (path.empty() || !path.HasParent())
+	if (path.empty() || !path.HasParent()) {
 		return;
+	}
 
 	const CServerPath& parent = path.GetParent();
 	const wxString& lastSegment = path.GetLastSegment();
-	if (lastSegment.empty())
+	if (lastSegment.empty()) {
 		return;
+	}
 
 	wxDataObjectComposite object;
 
-	CServer const* pServer = m_state.GetServer();
-	if (!pServer)
+	ServerWithCredentials server = m_state.GetServer(); // Make a copy as DoDragDrop later runs the event loop
+	if (!server) {
 		return;
-	CServer const server = *pServer;
+	}
 
-
-	CRemoteDataObject *pRemoteDataObject = new CRemoteDataObject(*pServer, parent);
+	CRemoteDataObject *pRemoteDataObject = new CRemoteDataObject(server, parent);
 	pRemoteDataObject->AddFile(lastSegment, true, -1, false);
 
 	pRemoteDataObject->Finalize();
@@ -826,7 +852,7 @@ void CRemoteTreeView::OnBeginDrag(wxTreeEvent& event)
 
 	CDragDropManager* pDragDropManager = CDragDropManager::Init();
 	pDragDropManager->pDragSource = this;
-	pDragDropManager->server = *pServer;
+	pDragDropManager->server = server;
 	pDragDropManager->remoteParent = parent;
 
 	wxDropSource source(this);
@@ -843,8 +869,8 @@ void CRemoteTreeView::OnBeginDrag(wxTreeEvent& event)
 #if FZ3_USESHELLEXT
 	if (ext) {
 		if (!pRemoteDataObject->DidSendData()) {
-			pServer = m_state.GetServer();
-			if (!pServer || !m_state.IsRemoteIdle() || *pServer != server) {
+			server = m_state.GetServer();
+			if (!server || !m_state.IsRemoteIdle() || server != server) {
 				wxBell();
 				return;
 			}
@@ -866,8 +892,9 @@ void CRemoteTreeView::OnContextMenu(wxTreeEvent& event)
 {
 	m_contextMenuItem = event.GetItem();
 	wxMenu* pMenu = wxXmlResource::Get()->LoadMenu(_T("ID_MENU_REMOTETREE"));
-	if (!pMenu)
+	if (!pMenu) {
 		return;
+	}
 
 	const CServerPath& path = m_contextMenuItem ? GetPathFromItem(m_contextMenuItem) : CServerPath();
 	if (!m_state.IsRemoteIdle() || path.empty()) {
@@ -1384,14 +1411,14 @@ void CRemoteTreeView::OnMenuGeturl(wxCommandEvent& event)
 		return;
 	}
 
-	const CServerPath& path = GetPathFromItem(m_contextMenuItem);
+	CServerPath const& path = GetPathFromItem(m_contextMenuItem);
 	if (path.empty()) {
 		wxBell();
 		return;
 	}
 
-	const CServer *pServer = m_state.GetServer();
-	if (!pServer) {
+	ServerWithCredentials const& server = m_state.GetServer();
+	if (!server) {
 		wxBell();
 		return;
 	}
@@ -1401,7 +1428,7 @@ void CRemoteTreeView::OnMenuGeturl(wxCommandEvent& event)
 		return;
 	}
 
-	wxString url = pServer->Format((event.GetId() == XRCID("ID_GETURL_PASSWORD")) ? ServerFormat::url_with_password : ServerFormat::url);
+	wxString url = server.Format((event.GetId() == XRCID("ID_GETURL_PASSWORD")) ? ServerFormat::url_with_password : ServerFormat::url);
 
 	std::wstring const pathPart = fz::percent_encode_w(path.GetPath(), true);
 	if (!pathPart.empty() && pathPart[0] != '/') {
