@@ -61,13 +61,13 @@ PqQuF7sJR6POArUVYkRD/2LIWsB7\n\
 
 void version_information::update_available()
 {
-	if( !nightly_.url_.empty() && COptions::Get()->GetOptionVal(OPTION_UPDATECHECK_CHECKBETA) == 2 ) {
+	if (!nightly_.url_.empty() && COptions::Get()->GetOptionVal(OPTION_UPDATECHECK_CHECKBETA) == 2) {
 		available_ = nightly_;
 	}
-	else if( !beta_.version_.empty() && COptions::Get()->GetOptionVal(OPTION_UPDATECHECK_CHECKBETA) != 0 ) {
+	else if (!beta_.version_.empty() && COptions::Get()->GetOptionVal(OPTION_UPDATECHECK_CHECKBETA) != 0) {
 		available_ = beta_;
 	}
-	else if( !stable_.version_.empty() ) {
+	else if (!stable_.version_.empty()) {
 		available_ = stable_;
 	}
 	else {
@@ -77,9 +77,10 @@ void version_information::update_available()
 
 static CUpdater* instance = 0;
 
-CUpdater::CUpdater(CUpdateHandler& parent, CFileZillaEngineContext& engine_context)
+CUpdater::CUpdater(CUpdateHandler& parent, CFileZillaEngineContext& engine_context, std::function<void(CActiveNotification const&)> const& activityNotificationHandler)
 	: state_(UpdaterState::idle)
 	, engine_(new CFileZillaEngine(engine_context, *this))
+	, activityNotificationHandler_(activityNotificationHandler)
 {
 	AddHandler(parent);
 }
@@ -90,7 +91,7 @@ void CUpdater::Init()
 		return;
 	}
 
-	raw_version_information_ = COptions::Get()->GetOption( OPTION_UPDATECHECK_NEWVERSION );
+	raw_version_information_ = COptions::Get()->GetOption(OPTION_UPDATECHECK_NEWVERSION);
 
 	UpdaterState s = ProcessFinishedData(FZ_AUTOUPDATECHECK);
 
@@ -101,16 +102,18 @@ void CUpdater::Init()
 	update_timer_.SetOwner(this);
 	update_timer_.Start(1000 * 3600);
 
-	if( !instance ) {
+	if (!instance) {
 		instance = this;
 	}
 }
 
 CUpdater::~CUpdater()
 {
-	if( instance == this ) {
-		instance  =0;
+	if (instance == this) {
+		instance = 0;
+	
 	}
+
 	delete engine_;
 }
 
@@ -122,8 +125,8 @@ CUpdater* CUpdater::GetInstance()
 void CUpdater::AutoRunIfNeeded()
 {
 #if FZ_AUTOUPDATECHECK
-	if( state_ == UpdaterState::failed || state_ == UpdaterState::idle ) {
-		if( !COptions::Get()->GetOptionVal(OPTION_DEFAULT_DISABLEUPDATECHECK) && COptions::Get()->GetOptionVal(OPTION_UPDATECHECK) != 0 && LongTimeSinceLastCheck() ) {
+	if (state_ == UpdaterState::failed || state_ == UpdaterState::idle) {
+		if (!COptions::Get()->GetOptionVal(OPTION_DEFAULT_DISABLEUPDATECHECK) && COptions::Get()->GetOptionVal(OPTION_UPDATECHECK) != 0 && LongTimeSinceLastCheck()) {
 			Run();
 		}
 	}
@@ -133,9 +136,9 @@ void CUpdater::AutoRunIfNeeded()
 void CUpdater::RunIfNeeded()
 {
 	build const b = AvailableBuild();
-	if( state_ == UpdaterState::idle || state_ == UpdaterState::failed ||
+	if (state_ == UpdaterState::idle || state_ == UpdaterState::failed ||
 		LongTimeSinceLastCheck() || (state_ == UpdaterState::newversion && !b.url_.empty()) ||
-		(state_ == UpdaterState::newversion_ready && !VerifyChecksum( DownloadedFile(), b.size_, b.hash_ ) ) )
+		(state_ == UpdaterState::newversion_ready && !VerifyChecksum(DownloadedFile(), b.size_, b.hash_)))
 	{
 		Run();
 	}
@@ -143,47 +146,53 @@ void CUpdater::RunIfNeeded()
 
 bool CUpdater::LongTimeSinceLastCheck() const
 {
-	wxString const lastCheckStr = COptions::Get()->GetOption(OPTION_UPDATECHECK_LASTDATE);
-	if (lastCheckStr.empty())
+	std::wstring const lastCheckStr = COptions::Get()->GetOption(OPTION_UPDATECHECK_LASTDATE);
+	if (lastCheckStr.empty()) {
 		return true;
+	}
 
-	fz::datetime lastCheck(lastCheckStr.ToStdWstring(), fz::datetime::utc);
-	if (lastCheck.empty())
+	fz::datetime lastCheck(lastCheckStr, fz::datetime::utc);
+	if (lastCheck.empty()) {
 		return true;
+	}
 
 	auto const span = fz::datetime::now() - lastCheck;
 
-	if (span.get_seconds() < 0)
+	if (span.get_seconds() < 0) {
 		// Last check in future
 		return true;
+	}
 
 	int days = 1;
-	if (!CBuildInfo::IsUnstable())
+	if (!CBuildInfo::IsUnstable()) {
 		days = COptions::Get()->GetOptionVal(OPTION_UPDATECHECK_INTERVAL);
+	}
 	return span.get_days() >= days;
 }
 
-wxString CUpdater::GetUrl()
+std::wstring CUpdater::GetUrl()
 {
-	wxString host = CBuildInfo::GetHostname();
-	if (host.empty())
-		host = _T("unknown");
+	std::wstring host = CBuildInfo::GetHostname();
+	if (host.empty()) {
+		host = L"unknown";
+	}
 
-	wxString version = CBuildInfo::GetVersion();
-	version.Replace(_T(" "), _T("%20"));
+	std::wstring version = CBuildInfo::GetVersion();
+	fz::replace_substrings(version, L" ", L"%20");
 
-	wxString url = wxString::Format(_T("https://update.filezilla-project.org/update.php?platform=%s&version=%s"), host, version);
+	std::wstring url = fz::sprintf(L"https://update.filezilla-project.org/update.php?platform=%s&version=%s", host, version);
 #if defined(__WXMSW__) || defined(__WXMAC__)
 	// Makes not much sense to submit OS version on Linux, *BSD and the likes, too many flavours.
-	wxString osVersion = wxString::Format(_T("&osversion=%d.%d"), wxPlatformInfo::Get().GetOSMajorVersion(), wxPlatformInfo::Get().GetOSMinorVersion());
-	url += osVersion;
+	url += fz::sprintf(L"&osversion=%d.%d", wxPlatformInfo::Get().GetOSMajorVersion(), wxPlatformInfo::Get().GetOSMinorVersion());
 #endif
 
 #ifdef __WXMSW__
-	if (wxIsPlatform64Bit())
-		url += _T("&osarch=64");
-	else
-		url += _T("&osarch=32");
+	if (wxIsPlatform64Bit()) {
+		url += L"&osarch=64";
+	}
+	else {
+		url += L"&osarch=32";
+	}
 
 	// Add information about package
 	{
@@ -198,27 +207,32 @@ wxString CUpdater::GetUrl()
 
 		long updated{};
 		if (key->GetValueType(_T("Updated")) == wxRegKey::Type_Dword && key->QueryValue(_T("Updated"), &updated)) {
-			url += wxString::Format(_T("&updated=%d"), updated);
+			url += fz::sprintf(L"&updated=%d", updated);
 		}
 
 		long package{};
 		if (key->GetValueType(_T("Package")) == wxRegKey::Type_Dword && key->QueryValue(_T("Package"), &package)) {
-			url += wxString::Format(_T("&package=%d"), package);
+			url += fz::sprintf(L"&package=%d", package);
+		}
+
+		wxString channel;
+		if (key->GetValueType(_T("Channel")) == wxRegKey::Type_String && key->QueryValue(_T("Channel"), channel)) {
+			url += fz::sprintf(L"&channel=%s", channel.ToStdWstring());
 		}
 	}
 #endif
 
-	wxString const cpuCaps = CBuildInfo::GetCPUCaps(',');
+	std::wstring const cpuCaps = CBuildInfo::GetCPUCaps(',');
 	if (!cpuCaps.empty()) {
-		url += _T("&cpuid=") + cpuCaps;
+		url += L"&cpuid=" + cpuCaps;
 	}
 
-	wxString const lastVersion = COptions::Get()->GetOption(OPTION_UPDATECHECK_LASTVERSION);
+	std::wstring const lastVersion = COptions::Get()->GetOption(OPTION_UPDATECHECK_LASTVERSION);
 	if (lastVersion != CBuildInfo::GetVersion()) {
-		url += _T("&initial=1");
+		url += L"&initial=1";
 	}
 	else {
-		url += _T("&initial=0");
+		url += L"&initial=0";
 	}
 
 	return url;
@@ -226,8 +240,8 @@ wxString CUpdater::GetUrl()
 
 bool CUpdater::Run()
 {
-	if( state_ != UpdaterState::idle && state_ != UpdaterState::failed &&
-		state_ != UpdaterState::newversion && state_ != UpdaterState::newversion_ready )
+	if (state_ != UpdaterState::idle && state_ != UpdaterState::failed &&
+		state_ != UpdaterState::newversion && state_ != UpdaterState::newversion_ready)
 	{
 		return false;
 	}
@@ -239,7 +253,7 @@ bool CUpdater::Run()
 	log_ = wxString::Format(_("Started update check on %s\n"), t.format(_T("%Y-%m-%d %H:%M:%S"), fz::datetime::local));
 
 	wxString build = CBuildInfo::GetBuildType();
-	if( build.empty() ) {
+	if (build.empty())  {
 		build = _("custom");
 	}
 	log_ += wxString::Format(_("Own build type: %s\n"), build);
@@ -257,7 +271,7 @@ bool CUpdater::Run()
 	return state_ == UpdaterState::checking;
 }
 
-int CUpdater::Download(wxString const& url, std::wstring const& local_file)
+int CUpdater::Download(std::wstring const& url, std::wstring const& local_file)
 {
 	wxASSERT(pending_commands_.empty());
 	pending_commands_.clear();
@@ -284,30 +298,30 @@ int CUpdater::ContinueDownload()
 	return res;
 }
 
-bool CUpdater::CreateConnectCommand(wxString const& url)
+bool CUpdater::CreateConnectCommand(std::wstring const& url)
 {
-	CServer s;
+	ServerWithCredentials s;
 	CServerPath path;
-	wxString error;
-	if( !s.ParseUrl( url, 0, wxString(), wxString(), error, path ) || (s.GetProtocol() != HTTP && s.GetProtocol() != HTTPS) ) {
+	std::wstring error;
+	if (!s.ParseUrl(url, 0, std::wstring(), std::wstring(), error, path) || (s.server.GetProtocol() != HTTP && s.server.GetProtocol() != HTTPS)) {
 		return false;
 	}
 
-	pending_commands_.emplace_back(new CConnectCommand(s));
+	pending_commands_.emplace_back(new CConnectCommand(s.server, s.credentials));
 	return true;
 }
 
-bool CUpdater::CreateTransferCommand(wxString const& url, std::wstring const& local_file)
+bool CUpdater::CreateTransferCommand(std::wstring const& url, std::wstring const& local_file)
 {
 	CFileTransferCommand::t_transferSettings transferSettings;
 
-	CServer s;
+	ServerWithCredentials s;
 	CServerPath path;
-	wxString error;
-	if (!s.ParseUrl(url, 0, wxString(), wxString(), error, path) || (s.GetProtocol() != HTTP && s.GetProtocol() != HTTPS)) {
+	std::wstring error;
+	if (!s.ParseUrl(url, 0, std::wstring(), std::wstring(), error, path) || (s.server.GetProtocol() != HTTP && s.server.GetProtocol() != HTTPS)) {
 		return false;
 	}
-	std::wstring file = path.GetLastSegment().ToStdWstring();
+	std::wstring file = path.GetLastSegment();
 	path = path.GetParent();
 
 	pending_commands_.emplace_back(new CFileTransferCommand(local_file, path, file, true, transferSettings));
@@ -321,17 +335,25 @@ void CUpdater::OnEngineEvent(CFileZillaEngine* engine)
 
 void CUpdater::DoOnEngineEvent(CFileZillaEngine* engine)
 {
-	if (!engine_ || engine_ != engine)
+	if (!engine_ || engine_ != engine) {
 		return;
+	}
 
 	std::unique_ptr<CNotification> notification;
-	while( (notification = engine_->GetNextNotification()) ) {
+	while ((notification = engine_->GetNextNotification())) {
 		ProcessNotification(std::move(notification));
 	}
 }
 
 void CUpdater::ProcessNotification(std::unique_ptr<CNotification> && notification)
 {
+	if (notification->GetID() == nId_active) {
+		if (activityNotificationHandler_) {
+			auto const& activityNotification = static_cast<CActiveNotification const&>(*notification.get());
+			activityNotificationHandler_(activityNotification);
+		}
+		return;
+	}
 	if (state_ != UpdaterState::checking && state_ != UpdaterState::newversion_downloading) {
 		return;
 	}
@@ -348,14 +370,12 @@ void CUpdater::ProcessNotification(std::unique_ptr<CNotification> && notificatio
 				auto & certNotification = static_cast<CCertificateNotification &>(*pData.get());
 				if (m_use_internal_rootcert) {
 					auto certs = certNotification.GetCertificates();
-					if( certs.size() > 1 ) {
-						auto ca = certs.back();
-
-						unsigned int ca_data_length{};
-						unsigned char const* ca_data = ca.GetRawData(ca_data_length);
+					if (certs.size() > 1) {
+						auto const& ca = certs.back();
+						std::vector<uint8_t> ca_data = ca.GetRawData();
 
 						std::string updater_root = fz::base64_decode(s_update_cert);
-						if (ca_data_length == updater_root.size() && !memcmp(ca_data, updater_root.c_str(), ca_data_length) ) {
+						if (ca_data.size() == updater_root.size() && !memcmp(&ca_data[0], updater_root.c_str(), ca_data.size())) {
 							certNotification.m_trusted = true;
 						}
 					}
@@ -415,7 +435,7 @@ UpdaterState CUpdater::ProcessFinishedData(bool can_download)
 				if (size >= 0 && size >= version_information_.available_.size_) {
 					s = ProcessFinishedDownload();
 				}
-				else if (!can_download || Download(version_information_.available_.url_, temp) != FZ_REPLY_WOULDBLOCK ) {
+				else if (!can_download || Download(version_information_.available_.url_, temp) != FZ_REPLY_WOULDBLOCK) {
 					s = UpdaterState::newversion;
 				}
 			}
@@ -457,7 +477,7 @@ void CUpdater::ProcessOperation(COperationNotification const& operation)
 		}
 	}
 	else if (state_ == UpdaterState::checking) {
-		COptions::Get()->SetOption(OPTION_UPDATECHECK_LASTVERSION, CBuildInfo::GetVersion().ToStdWstring());
+		COptions::Get()->SetOption(OPTION_UPDATECHECK_LASTVERSION, CBuildInfo::GetVersion());
 		s = ProcessFinishedData(true);
 	}
 	else {
@@ -485,9 +505,9 @@ UpdaterState CUpdater::ProcessFinishedDownload()
 		wxString local_file = GetLocalFile(version_information_.available_, false);
 
 		wxLogNull log;
-		if (local_file.empty() || !wxRenameFile( temp, local_file, false ) ) {
+		if (local_file.empty() || !wxRenameFile(temp, local_file, false)) {
 			s = UpdaterState::newversion;
-			wxRemoveFile( temp );
+			wxRemoveFile(temp);
 			log_ += wxString::Format(_("Could not create local file %s\n"), local_file);
 		}
 		else {
@@ -500,7 +520,7 @@ UpdaterState CUpdater::ProcessFinishedDownload()
 
 wxString CUpdater::GetLocalFile(build const& b, bool allow_existing)
 {
-	wxString const fn = GetFilename( b.url_ );
+	wxString const fn = GetFilename(b.url_);
 	wxString const dl = GetDownloadDir().GetPath();
 
 	int i = 1;
@@ -532,26 +552,26 @@ wxString CUpdater::GetLocalFile(build const& b, bool allow_existing)
 
 void CUpdater::ProcessData(CDataNotification& dataNotification)
 {
-	if( state_ != UpdaterState::checking ) {
+	if (state_ != UpdaterState::checking) {
 		return;
 	}
 
 	int len;
 	char* data = dataNotification.Detach(len);
 
-	if( COptions::Get()->GetOptionVal(OPTION_LOGGING_DEBUGLEVEL) == 4 ) {
+	if (COptions::Get()->GetOptionVal(OPTION_LOGGING_DEBUGLEVEL) == 4) {
 		log_ += wxString::Format(_T("ProcessData %d\n"), len);
 	}
 
-	if( raw_version_information_.size() + len > 0x40000) {
-		log_ += _("Received version information is too large");
+	if (raw_version_information_.size() + len > 0x40000) {
+		log_ += _("Received version information is too large") + L"\n";
 		engine_->Cancel();
 		SetState(UpdaterState::failed);
 	}
 	else {
 		for (int i = 0; i < len; ++i) {
 			if (data[i] < 10 || (unsigned char)data[i] > 127) {
-				log_ += _("Received invalid character in version information");
+				log_ += _("Received invalid character in version information") + L"\n";
 				SetState(UpdaterState::failed);
 				engine_->Cancel();
 				break;
@@ -559,7 +579,7 @@ void CUpdater::ProcessData(CDataNotification& dataNotification)
 		}
 	}
 
-	if( state_ == UpdaterState::checking ) {
+	if (state_ == UpdaterState::checking) {
 		raw_version_information_ += wxString(data, wxConvUTF8, len);
 	}
 	delete [] data;
@@ -574,7 +594,7 @@ void CUpdater::ParseData()
 
 	log_ += wxString::Format(_("Parsing %d bytes of version information.\n"), static_cast<int>(raw_version_information.size()));
 
-	while( !raw_version_information.empty() ) {
+	while (!raw_version_information.empty()) {
 		wxString line;
 		int pos = raw_version_information.Find('\n');
 		if (pos != -1) {
@@ -586,7 +606,7 @@ void CUpdater::ParseData()
 			raw_version_information.clear();
 		}
 
-		wxStringTokenizer tokens(line, _T(" \t\n"),  wxTOKEN_STRTOK);
+		wxStringTokenizer tokens(line, _T(" \t\n"), wxTOKEN_STRTOK);
 		if (!tokens.CountTokens()) {
 			// After empty line, changelog follows
 			version_information_.changelog_ = raw_version_information;
@@ -629,7 +649,7 @@ void CUpdater::ParseData()
 
 			fz::datetime buildDate = CBuildInfo::GetBuildDate();
 			if (buildDate.empty() || nightlyDate.empty() || nightlyDate <= buildDate) {
-				if( COptions::Get()->GetOptionVal(OPTION_LOGGING_DEBUGLEVEL) == 4 ) {
+				if (COptions::Get()->GetOptionVal(OPTION_LOGGING_DEBUGLEVEL) == 4) {
 					log_ += _T("Nightly isn't newer\n");
 				}
 				continue;
@@ -637,57 +657,67 @@ void CUpdater::ParseData()
 		}
 		else {
 			int64_t v = CBuildInfo::ConvertToVersionNumber(versionOrDate.c_str());
-			if (v <= ownVersionNumber)
+			if (v <= ownVersionNumber) {
 				continue;
-		}
-
-		build* b = 0;
-		if( type == _T("nightly") && UpdatableBuild() ) {
-			b = &version_information_.nightly_;
-		}
-		else if( type == _T("release") ) {
-			b = &version_information_.stable_;
-		}
-		else if( type == _T("beta") ) {
-			b = &version_information_.beta_;
-		}
-
-		if( b ) {
-			b->version_ = versionOrDate;
-
-			if( UpdatableBuild() && tokens.CountTokens() == 4 ) {
-				wxString const url = tokens.GetNextToken();
-				wxString const sizestr = tokens.GetNextToken();
-				wxString const hash_algo = tokens.GetNextToken();
-				wxString const hash = tokens.GetNextToken();
-
-				if( GetFilename(url).empty() ) {
-					if( COptions::Get()->GetOptionVal(OPTION_LOGGING_DEBUGLEVEL) == 4 ) {
-						log_ += wxString::Format(_T("Could not extract filename from URL: %s\n"), url);
-					}
-					continue;
-				}
-
-				if( hash_algo.CmpNoCase(_T("sha512")) ) {
-					continue;
-				}
-
-				unsigned long long l = 0;
-				if( !sizestr.ToULongLong(&l) ) {
-					if( COptions::Get()->GetOptionVal(OPTION_LOGGING_DEBUGLEVEL) == 4 ) {
-						log_ += wxString::Format(_T("Could not parse size: %s"), sizestr);
-					}
-					continue;
-				}
-
-				b->url_ = url;
-				b->size_ = l;
-				b->hash_ = hash;
-
-				// @translator: Two examples: Found new nightly 2014-04-03\n, Found new release 3.9.0.1\n
-				log_ += wxString::Format(_("Found new %s %s\n"), type, b->version_);
 			}
 		}
+
+		build b;
+		b.version_ = versionOrDate;
+
+		if (UpdatableBuild() && tokens.CountTokens() == 4) {
+			wxString const url = tokens.GetNextToken();
+			wxString const sizestr = tokens.GetNextToken();
+			wxString const hash_algo = tokens.GetNextToken();
+			wxString const hash = tokens.GetNextToken();
+
+			if (GetFilename(url).empty()) {
+				if (COptions::Get()->GetOptionVal(OPTION_LOGGING_DEBUGLEVEL) == 4) {
+					log_ += wxString::Format(_T("Could not extract filename from URL: %s\n"), url);
+				}
+				continue;
+			}
+
+			if (hash_algo.CmpNoCase(_T("sha512"))) {
+				continue;
+			}
+
+			unsigned long long l = 0;
+			if (!sizestr.ToULongLong(&l)) {
+				if (COptions::Get()->GetOptionVal(OPTION_LOGGING_DEBUGLEVEL) == 4) {
+					log_ += wxString::Format(_T("Could not parse size: %s"), sizestr) + L"\n";
+				}
+				continue;
+			}
+
+			b.url_ = url;
+			b.size_ = l;
+			b.hash_ = fz::str_tolower_ascii(hash.ToStdWstring());
+			bool valid_hash = true;
+			for (auto const& c : b.hash_) {
+				if ((c < 'a' || c > 'f') && (c < '0' || c > '9')) {
+					valid_hash = false;
+					break;
+				}
+			}
+			if (!valid_hash) {
+				log_ += wxString::Format(_("Invalid hash: %s\n"), hash);
+				continue;
+			}
+
+			// @translator: Two examples: Found new nightly 2014-04-03\n, Found new release 3.9.0.1\n
+			log_ += wxString::Format(_("Found new %s %s\n"), type, b.version_);
+		}
+
+		if (type == _T("nightly") && UpdatableBuild()) {
+			version_information_.nightly_ = b;
+		}
+		else if (type == _T("release")) {
+			version_information_.stable_ = b;
+		}
+		else if (type == _T("beta")) {
+			version_information_.beta_ = b;
+		}
 	}
 
 	version_information_.update_available();
@@ -700,14 +730,19 @@ void CUpdater::OnTimer(wxTimerEvent&)
 	AutoRunIfNeeded();
 }
 
-bool CUpdater::VerifyChecksum(wxString const& file, int64_t size, wxString const& checksum)
+bool CUpdater::VerifyChecksum(wxString const& file, int64_t size, std::wstring const& checksum)
 {
 	if (file.empty() || checksum.empty()) {
 		return false;
 	}
 
 	auto filesize = fz::local_filesys::get_size(fz::to_native(file));
-	if (filesize < 0 || filesize != size) {
+	if (filesize < 0) {
+		log_ += wxString::Format(_("Could not obtain size of '%s'"), file) + L"\n";
+		return false;
+	}
+	else if (filesize != size) {
+		log_ += fz::sprintf(_("Local size of '%s' does not match expected size: %d != %d"), file, filesize, size) + L"\n";
 		return false;
 	}
 
@@ -717,6 +752,7 @@ bool CUpdater::VerifyChecksum(wxString const& file, int64_t size, wxString const
 	{
 		fz::file f(fz::to_native(file), fz::file::reading);
 		if (!f.opened()) {
+			log_ += wxString::Format(_("Could not open '%s'"), file) + L"\n";
 			return false;
 		}
 		unsigned char buffer[65536];
@@ -725,30 +761,18 @@ bool CUpdater::VerifyChecksum(wxString const& file, int64_t size, wxString const
 			sha512_update(&state, static_cast<size_t>(read), buffer);
 		}
 		if (read < 0) {
+			log_ += wxString::Format(_("Could not read from '%s'"), file) + L"\n";
 			return false;
 		}
 	}
 
-	unsigned char raw_digest[64];
-	sha512_digest(&state, 64, raw_digest);
+	std::string raw_digest;
+	raw_digest.resize(64);
+	sha512_digest(&state, 64, reinterpret_cast<uint8_t*>(&raw_digest[0]));
 
-	wxString digest;
-	for (unsigned int i = 0; i < sizeof(raw_digest); ++i) {
-		unsigned char l = raw_digest[i] >> 4;
-		unsigned char r = raw_digest[i] & 0x0F;
+	auto const digest = fz::hex_encode<std::wstring>(raw_digest);
 
-		if (l > 9)
-			digest += 'a' + l - 10;
-		else
-			digest += '0' + l;
-
-		if (r > 9)
-			digest += 'a' + r - 10;
-		else
-			digest += '0' + r;
-	}
-
-	if (checksum.CmpNoCase(digest)) {
+	if (digest != checksum) {
 		log_ += wxString::Format(_("Checksum mismatch on file %s\n"), file);
 		return false;
 	}
@@ -759,20 +783,20 @@ bool CUpdater::VerifyChecksum(wxString const& file, int64_t size, wxString const
 
 std::wstring CUpdater::GetTempFile() const
 {
-	wxASSERT( !version_information_.available_.hash_.empty() );
-	wxString ret = wxFileName::GetTempDir();
+	wxASSERT(!version_information_.available_.hash_.empty());
+	std::wstring ret = wxFileName::GetTempDir().ToStdWstring();
 	if (!ret.empty()) {
-		if (ret.Last() != wxFileName::GetPathSeparator()) {
+		if (ret.back() != wxFileName::GetPathSeparator()) {
 			ret += wxFileName::GetPathSeparator();
 		}
 
-		ret += _T("fzupdate_") + version_information_.available_.hash_.Left(16) + _T(".tmp");
+		ret += L"fzupdate_" + version_information_.available_.hash_.substr(0, 16) + L".tmp";
 	}
 
-	return ret.ToStdWstring();
+	return ret;
 }
 
-wxString CUpdater::GetFilename( wxString const& url) const
+wxString CUpdater::GetFilename(wxString const& url) const
 {
 	wxString ret;
 	int pos = url.Find('/', true);
@@ -780,7 +804,7 @@ wxString CUpdater::GetFilename( wxString const& url) const
 		ret = url.Mid(pos + 1);
 	}
 	size_t p = ret.find_first_of(_T("?#"));
-	if( p != std::string::npos ) {
+	if (p != std::string::npos) {
 		ret = ret.substr(0, p);
 	}
 #ifdef __WXMSW__
@@ -790,18 +814,18 @@ wxString CUpdater::GetFilename( wxString const& url) const
 	return ret;
 }
 
-void CUpdater::SetState( UpdaterState s )
+void CUpdater::SetState(UpdaterState s)
 {
-	if( s != state_ ) {
+	if (s != state_) {
 		state_ = s;
 
 		if (s != UpdaterState::checking && s != UpdaterState::newversion_downloading) {
 			pending_commands_.clear();
 		}
 		build b = version_information_.available_;
-		for (auto const& handler : handlers_ ) {
-			if( handler ) {
-				handler->UpdaterStateChanged( s, b );
+		for (auto const& handler : handlers_) {
+			if (handler) {
+				handler->UpdaterStateChanged(s, b);
 			}
 		}
 	}
@@ -810,21 +834,21 @@ void CUpdater::SetState( UpdaterState s )
 wxString CUpdater::DownloadedFile() const
 {
 	wxString ret;
-	if( state_ == UpdaterState::newversion_ready ) {
+	if (state_ == UpdaterState::newversion_ready) {
 		ret = local_file_;
 	}
 	return ret;
 }
 
-void CUpdater::AddHandler( CUpdateHandler& handler )
+void CUpdater::AddHandler(CUpdateHandler& handler)
 {
-	for( auto const& h : handlers_ ) {
+	for (auto const& h : handlers_) {
 		if (h == &handler) {
 			return;
 		}
 	}
-	for( auto& h : handlers_ ) {
-		if( !h ) {
+	for (auto& h : handlers_) {
+		if (!h) {
 			h = &handler;
 			return;
 		}
@@ -832,7 +856,7 @@ void CUpdater::AddHandler( CUpdateHandler& handler )
 	handlers_.push_back(&handler);
 }
 
-void CUpdater::RemoveHandler( CUpdateHandler& handler )
+void CUpdater::RemoveHandler(CUpdateHandler& handler)
 {
 	for (auto& h : handlers_) {
 		if (h == &handler) {
